/////////////////////////////////////////////////////////////////////////////
// Name:        No names yet.
// Purpose:     Contrib. demo
// Author:      Aleksandras Gluchovas
// Modified by:
// Created:     2000/02/08
// RCS-ID:      
// Copyright:   (c) Aleksandras Gluchovas
// Licence:   	GPL (version 2.0 or later)
/////////////////////////////////////////////////////////////////////////////

#ifndef __PM_G__
#define __PM_G__

#include "wx/pmf/pm_cmn.h"
#include "wx/pmf/pm_doc.h"
#include "wx/pmf/pm_docmgr.h"

class wxPMMenuItemData : public wxObject
{
public:
	wxPMMenuItemData() : m_pReceiver( NULL ),
						 m_ActualId(-1)
		{}
public:
	wxEvtHandler*         m_pReceiver;
	int                   m_ActualId;
};

// Listener intented to receive service-management related
// notifications, those can occure even when the workplace 
// is not opened yet

class wxPMBootstrapServiceListener
{
public:
	virtual void OnServiceStarted( wxPMService& service );
	virtual void OnServiceStopped( wxPMService& service );

	virtual void OnPaneShown( wxPMPane& pane );
	virtual void OnPaneHidden( wxPMPane& pane );
};

/*
 * misc. NOTES: bootstrap is a special service is, the one which is 
 * always availabe, even if the workplace is not opened. It also
 * takes over menu-merging functionality, since it is needed
 * from the very start. Currently, the menu-merging alg. does not make 
 * use of wxMenu::Delete method, because (AFAIK) wxMenu/wxMenubar does not 
 * allow removing of top-level menus, and with it's API i cannot *insert* a 
 * single item at particular position a in top-level or -submenu. Instead, 
 * the alg. mirrors the entire tree of menus as in-memory structures, 
 * and uses them to recreate menu-bar whenever any changes or merges (even small 
 * ones) occure. TODO:: ask wx-developers...
 */

class wxPMBootstrapService : public wxEvtHandler, public wxPMService
{
public:
	wxPMBootstrapService( wxDocMDIParentFrame* pMainWnd, wxPMDocManager* pDocMgr );
	~wxPMBootstrapService();

	// closes all documents, removes related objects and releases itself
	// returns FALSE, if the any of the closure was veto'ed

	virtual bool CleanUp();

	virtual wxDocMDIParentFrame* GetMainWindow();
	virtual wxKeyBinder& GetKeyBinder();
	virtual void TranslateKeyEvent( wxKeyEvent& event );

	/// overriden methods of wxPMService ///
	virtual bool Start();
	virtual bool Stop();
	virtual wxPMMergableMenuChain* GetMergableMenuChain();
	virtual wxObject* GetWxObject();

	virtual void SetStatusText( const wxString& message );

	////////////////////////////////////////

	virtual void AddListener( wxPMBootstrapServiceListener* pListener );
	virtual void RemoveListener( wxPMBootstrapServiceListener* pListener );

	// sets currently opened workplace
	virtual void SetWorkplace( wxPMWorkplace* wp );
	virtual wxPMWorkplace* GetWorkplace();

	virtual bool HasWorkplace() { return m_pWorkplace != NULL; }

	virtual wxPMDocManager* GetDocManager();

	// service activaiton/disposal
	virtual void AddService( wxPMService* pSvc );
	virtual void StartService( wxPMService* pSvc );
	virtual wxPMService* FindServiceByClass( wxClassInfo* pExactSvcClass );
	virtual void StopService( wxPMService* pSvc );
	virtual void StopAllServices();
	virtual void RemoveAllServices();
	virtual void RemoveService( wxPMService* pSvc );
	virtual void DetachService( wxPMService* pSvc );
	virtual void FindServicesByClass( wxClassInfo* pSvcClass,
									  PMServiceListT& toLst, bool machSubclasses );

	// displaying/hiding of panes
	virtual void ShowPane( wxPMPane* pPane );
	virtual void HidePane( wxPMPane* pPane );

	// meriging of menus into menubar of main window
	virtual void RemoveMenuChains( wxPMMergableMenuChain* pChains );
	virtual void MergeMenuChains( wxPMMergableMenuChain* pChains, wxEvtHandler* pOwner );

	// returns number of services actually removed
	virtual int RemoveServicesByClass( wxClassInfo* pSvcClass );
	virtual bool StartServicesByClass( wxClassInfo* pSvcClass );

	virtual void SetAppTitle( const wxString& appTitle );
	virtual wxString& GetAppTitle();

	// called by wxPMDocManager
	virtual void ActiaveteRelatedPanes( wxPMDocument* pDoc, bool activate );

	// event handlers

	void OnIdle( wxIdleEvent& event );      // used for updating menu-bar (batches multiple
											// menu-merges/removes into one to reduce flicker)

	void OnMenuEvent( wxEvent& event );	    // used for translation of event-ids to those
									       	// of "requested" by event-owners
	void OnButtonEvent( wxEvent& event );   // -/-
	void OnUpdateUIEvent( wxEvent& event ); // -/-

	/*** command-event handlers ***/

	void OnExit( wxCommandEvent& event );
	void OnAbout( wxCommandEvent& event );
	void OnTestKeyBindings( wxCommandEvent& event );

	// IDs which can be used by other services, for
	// translation of command-events to the boot-strap object

	enum COMMAND_IDS
	{
		ID_BOOTSTRAP_ON_EXIT = 20000, // it's because events of
									  // bootstrap svc. are handled directly
									  // (without translation)
		ID_BOOTSTRAP_ON_ABOUT,
		ID_BOOTSTRAP_TEST_KEYBINDINGS
	};

	DECLARE_EVENT_TABLE()

protected:
	virtual void DoTranslateEvent( wxEvent& event );
	virtual wxCommandInfo* FindItemByName( const wxString& path );
	virtual wxCommandInfo* DoFindItemByName( const char* path, wxCommandInfoListT& lst );
	virtual void DoRemoveChain( wxPMMergableMenuChain* pChain );
	virtual void DoMergeChain( wxPMMergableMenuChain* pChain,
							   wxEvtHandler* pReceiver );
	virtual void MutateCommandToGroup( wxCommandInfoListT& lst, size_t cmdIdx );
	virtual wxCommandInfoListT* CreateTreePath( const wxString& path, wxCommandInfo*& pParent );
	virtual wxString StripLastPathItem( const wxString& treePath );
	virtual wxString GetPathItem( const wxString& treePath, size_t i );
	virtual void RemoveItemByPath( const wxString& treePath );
	virtual void DoRegenerateteCommandIds( wxCommandInfoListT& commands, int& nextId );
	virtual void RegenerateteCommandIds();

protected:
	wxPMWorkplace*		 m_pWorkplace; // openned workplace, NULL otherwise
	wxDocMDIParentFrame* m_pMainWnd;	  // not owned by bootstrap. svc
	wxPMDocManager*		 m_pDocMgr;	  // -/-

	PMServiceListT		 m_Services;	 // released outmatically when workplace is closed

	wxKeyBinder			 m_KeyBinder; // mirrors hierarchy of native menu items
									    // to which have not access with current wxWin API

	bool m_RecreatePending; // is set whenever content of menu-tree is changed,
						   // cleared when EVT_IDLE occures where menubar is recreated

	wxString			m_AppTitle;
	wxHashTable			m_MenuItemsHash;

	PMBootstrapServiceListenerListT m_Listeners;
};

/* global  access-point to the single instance of bootstrap service */

void wxSetPMBootstrapSvc( wxPMBootstrapService* pSvc );

wxPMBootstrapService& wxGetPMBootstrapSvc();
wxPMBootstrapService& wxPMBootSvc(); // shorter-synonym

#endif // __PM_G__