/////////////////////////////////////////////////////////////////////////////
// Name:        No names yet.
// Purpose:     Contrib. demo
// Author:      Aleksandras Gluchovas
// Modified by:
// Created:     2000/02/08
// RCS-ID:      
// Copyright:   (c) Aleksandras Gluchovas
// Licence:   	GPL (version 2.0 or later)
/////////////////////////////////////////////////////////////////////////////

#ifdef __GNUG__
    #pragma implementation "pm.cpp"
    #pragma interface "pm.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include "wx/pmf/pm.h"
#include "winsvc.h"

/***** Implementation for class wxPMBootstrapServiceListener *****/

void wxPMBootstrapServiceListener::OnServiceStarted( wxPMService& service )
{}

void wxPMBootstrapServiceListener::OnServiceStopped( wxPMService& service )
{}

void wxPMBootstrapServiceListener::OnPaneShown( wxPMPane& pane )
{}

void wxPMBootstrapServiceListener::OnPaneHidden( wxPMPane& pane )
{}


/***** Implementation for class wxPMBootstrapService *****/

// TBD:: should be made statics of wxPMBootstrapService class

// mergable menus for bootstrap service

static wxCommandInfo _gBootstrapCommands[] = {

	wxCommandInfo( -1, "SEPARATOR" ),
	wxCommandInfo( wxPMBootstrapService::ID_BOOTSTRAP_ON_ABOUT, "Abou&t" ), 
	wxCommandInfo( -1, "SEPARATOR" ),
	wxCommandInfo( wxPMBootstrapService::ID_BOOTSTRAP_ON_EXIT, "E&xit" ), 

	wxCommandInfo( 0, "" ) // terminator
};

static wxCommandInfo _gBootstrapCommands1[] = {

	wxCommandInfo( wxPMBootstrapService::ID_BOOTSTRAP_TEST_KEYBINDINGS, "&Keyboard bindings..." ), 

	wxCommandInfo( 0, "" ) // terminator
};


static wxPMMergableMenuChain _gBootstrapChains[] = {

	wxPMMergableMenuChain( "/&File/",  _gBootstrapCommands ),  // first chain
	wxPMMergableMenuChain( "/Customi&ze/", _gBootstrapCommands1 ), // second chain

	wxPMMergableMenuChain( "", NULL ) // terminator
};

BEGIN_EVENT_TABLE( wxPMBootstrapService, wxEvtHandler )

	EVT_CUSTOM_RANGE( wxEVT_COMMAND_MENU_SELECTED,  500, 5000, wxPMBootstrapService::OnMenuEvent )
	EVT_CUSTOM_RANGE( wxEVT_COMMAND_BUTTON_CLICKED, 500, 5000, wxPMBootstrapService::OnButtonEvent )
	EVT_CUSTOM_RANGE( wxEVT_UPDATE_UI, 500, 5000, wxPMBootstrapService::OnUpdateUIEvent )

	EVT_MENU( wxPMBootstrapService::ID_BOOTSTRAP_ON_EXIT,  wxPMBootstrapService::OnExit )
	EVT_MENU( wxPMBootstrapService::ID_BOOTSTRAP_ON_ABOUT, wxPMBootstrapService::OnAbout )
	EVT_MENU( wxPMBootstrapService::ID_BOOTSTRAP_TEST_KEYBINDINGS, wxPMBootstrapService::OnTestKeyBindings )

	EVT_IDLE( wxPMBootstrapService::OnIdle )

END_EVENT_TABLE()

wxPMBootstrapService::wxPMBootstrapService( wxDocMDIParentFrame* pMainWnd, 
										    wxPMDocManager* pDocMgr )

	: m_pWorkplace( NULL ),
	  m_pMainWnd( pMainWnd ),
	  m_pDocMgr( pDocMgr ),
	  m_RecreatePending( FALSE ),
	  m_AppTitle( "Default-PMF-Application-Title" )
{
	SetRefData( (wxObjectRefData*)(wxPMService*)this );

	wxSetPMBootstrapSvc( this );
}

wxPMBootstrapService::~wxPMBootstrapService()
{
	if ( &wxPMBootSvc() != NULL )

		CleanUp();

	SetRefData( NULL );
}

bool wxPMBootstrapService::CleanUp()
{
	if ( this->IsStarted() && !this->Stop() )

		return FALSE; // veto!

	RemoveAllServices();

	wxSetPMBootstrapSvc( NULL );

	//if ( m_pDocMgr ) delete m_pDocMgr;
	m_pDocMgr = NULL;

	if ( &wxPMBmpMgr() != NULL )
	{
		delete &wxPMBmpMgr();

		wxSetPMBitmapManager( NULL );
	}

	return TRUE;
}

wxDocMDIParentFrame* wxPMBootstrapService::GetMainWindow()
{
	return m_pMainWnd;
}

wxKeyBinder& wxPMBootstrapService::GetKeyBinder()
{
	return m_KeyBinder;
}

void wxPMBootstrapService::TranslateKeyEvent( wxKeyEvent& event )
{
	m_KeyBinder.OnChar( event );
}

wxPMDocManager* wxPMBootstrapService::GetDocManager()
{
	return m_pDocMgr;
}

bool wxPMBootstrapService::Start()
{
	wxASSERT( this->IsStarted() == FALSE ); // DBG::

	wxASSERT_MSG( m_pMainWnd, "SetMainWindow() should be called before wxPMBootstrapService::Init()!" );

	wxASSERT_MSG( m_pDocMgr, "PM-doc-manager should be set before starting wxPMBootstrapService!" );

	// event receiver chain:
	//
	//	first:	 wxKeyBinder
	//	then:	 wxPMBootstrapService
	//  finally: wxWindow (app's main window)

#if 0
	m_pMainWnd->SetEventHandler( &m_KeyBinder );
	m_KeyBinder.SetNextHandler( this );
	this->SetNextHandler( m_pMainWnd );
#else
	//m_pMainWnd->PushEventHandler( &m_KeyBinder );
	m_pMainWnd->PushEventHandler( this );
	m_KeyBinder.SetNextHandler( this );
#endif

	MergeMenuChains( m_pDocMgr->GetMergableMenuChain(), m_pDocMgr );

	MergeMenuChains( this->GetMergableMenuChain(), this );

	this->SetIsStarted( TRUE );

	return TRUE;
}

bool wxPMBootstrapService::Stop()
{
	wxASSERT( this->IsStarted() ); // DBG::

	if ( HasWorkplace() )
	{
		if ( !m_pDocMgr->CloseWorkplace() )

			return FALSE; // veto Stop() request
	}

	StopAllServices();

	if ( m_pMainWnd ) 
	{
		m_pMainWnd->PopEventHandler(); // first pop keybinder

		m_pMainWnd->SetEventHandler( m_pMainWnd ); // then remove ourselves from chain
	}

	RemoveMenuChains( this->GetMergableMenuChain() );

	this->SetIsStarted( FALSE );

	return TRUE;
}

wxPMMergableMenuChain* wxPMBootstrapService::GetMergableMenuChain()
{
	return _gBootstrapChains;
}

wxObject* wxPMBootstrapService::GetWxObject()
{
	return (wxObject*)this;
}

void wxPMBootstrapService::SetStatusText( const wxString& message )
{
	if ( m_pMainWnd->GetStatusBar() )

		m_pMainWnd->GetStatusBar()->SetStatusText( message );
}

void wxPMBootstrapService::AddListener( wxPMBootstrapServiceListener* pListener )
{
	m_Listeners.Add( pListener );
}

void wxPMBootstrapService::RemoveListener( wxPMBootstrapServiceListener* pListener )
{
	m_Listeners.Remove( pListener );
}

void wxPMBootstrapService::SetWorkplace( wxPMWorkplace* wp )
{
	m_pWorkplace = wp;
}

wxPMWorkplace* wxPMBootstrapService::GetWorkplace()
{
	return m_pWorkplace;
}
								
void wxPMBootstrapService::AddService( wxPMService* pSvc )
{
	m_Services.Add( pSvc );
}

wxPMService* wxPMBootstrapService::FindServiceByClass( wxClassInfo* pExactSvcClass )
{
	for( size_t i = 0;i != m_Services.Count(); ++i )

		if ( m_Services[i]->GetWxObject()->GetClassInfo() == pExactSvcClass &&
			 m_Services[i]->IsStarted()
		   )

			return m_Services[i];

	return NULL;
}

void wxPMBootstrapService::StartService( wxPMService* pSvc )
{
	wxObject* pObj = pSvc->GetWxObject();

	// only wxEvtHandler-derivatives may receive events and thuse
	// have their own menus added

	if ( pObj->IsKindOf( CLASSINFO(wxEvtHandler) ) 
	   )
	{
		MergeMenuChains( pSvc->GetMergableMenuChain(), (wxEvtHandler*)pObj );
	}

	pSvc->SetIsStarted( TRUE );
	pSvc->Start();

	if( pSvc->GetWxObject()->IsKindOf( CLASSINFO( wxPMPane ) ) )
	{
		wxPMPane* pPane = (wxPMPane*)pSvc->GetWxObject();

		if ( pPane->GetDocumentTypes() == NULL )

			ShowPane( pPane );
	}

	wxPM_NOTIFY_ONE_ARG( OnServiceStarted, *pSvc );
}

void wxPMBootstrapService::StopService( wxPMService* pSvc )
{
	wxObject* pObj = pSvc->GetWxObject();

	if ( pObj->IsKindOf( CLASSINFO(wxEvtHandler) ) 
	   )
	
		RemoveMenuChains( pSvc->GetMergableMenuChain() );

	if( pSvc->GetWxObject()->IsKindOf( CLASSINFO( wxPMPane ) ) )
	{
		wxPMPane* pPane = (wxPMPane*)pSvc->GetWxObject();

		if ( pPane->IsShown() )

			HidePane( pPane );
	}

	pSvc->Stop();
	pSvc->SetIsStarted( FALSE );

	wxPM_NOTIFY_ONE_ARG( OnServiceStopped, *pSvc );
}

void wxPMBootstrapService::StopAllServices()
{
	for( size_t i = 0; i != m_Services.Count(); ++i )

		if ( m_Services[i]->IsStarted() )

			StopService( m_Services[i] );
}

void wxPMBootstrapService::RemoveAllServices()
{
	for( size_t i = 0; i != m_Services.Count(); ++i )
	{
		if ( m_Services[i]->IsStarted() )

			m_Services[i]->Stop();

		delete m_Services[i]->GetWxObject();
	}

	m_Services.Clear();
}

void wxPMBootstrapService::RemoveService( wxPMService* pSvc )
{
	delete pSvc->GetWxObject();
	m_Services.Remove( pSvc );
}

void wxPMBootstrapService::DetachService( wxPMService* pSvc )
{
	m_Services.Remove( pSvc );
}

void wxPMBootstrapService::FindServicesByClass( wxClassInfo* pSvcClass,
											    PMServiceListT& toLst, 
												bool machSubclasses )
{
	for( size_t i = 0; i != m_Services.Count(); ++i )
	{
		wxPMService* pSvc = m_Services[i];
		wxObject* pObj = pSvc->GetWxObject();

		bool match = ( machSubclasses ) ? pObj->IsKindOf( pSvcClass )
									    : pObj->GetClassInfo() == pSvcClass;

		if ( match ) toLst.Add( pSvc );
	}
}

void wxPMBootstrapService::ShowPane( wxPMPane* pPane )
{
	pPane->SetShown( TRUE );

	wxPM_NOTIFY_ONE_ARG( OnPaneShown, *pPane );
}

void wxPMBootstrapService::HidePane( wxPMPane* pPane )
{
	pPane->SetShown( FALSE );

	wxPM_NOTIFY_ONE_ARG( OnPaneHidden, *pPane );
}
	 
void wxPMBootstrapService::RemoveMenuChains( wxPMMergableMenuChain* pChains )
{
	if ( pChains == NULL ) 
        return;

	while( pChains->m_pCommands != NULL )

		DoRemoveChain( pChains++ );

	m_RecreatePending = TRUE;
}

void wxPMBootstrapService::MergeMenuChains( wxPMMergableMenuChain* pChains, wxEvtHandler* pOwner )
{
	if ( pChains == NULL ) return;

	while( pChains->m_pCommands != NULL )

		DoMergeChain( pChains++, pOwner );

	m_RecreatePending = TRUE;
}

/*** event handlers ***/

void wxPMBootstrapService::OnExit( wxCommandEvent& event )
{
	Stop();

	m_pMainWnd->Destroy();

	delete this;
}

void wxPMBootstrapService::OnAbout( wxCommandEvent& event )
{
	wxMessageBox( "wxWorkshop-0.3 demo\n"
				  "2000 (C) Alekandras Gluchovas (email:alex@soften.ktu.lt)\n" 
				  "HTTP://soften.ktu.lt/~alex/yeahhh"
				);
}

void wxPMBootstrapService::OnTestKeyBindings( wxCommandEvent& event )
{
	wxKeyConfigDialog dlg( GetMainWindow(), &m_KeyBinder );
/* glt
    int w,h;
	dlg.GetClientSize( &w, &h );
	dlg.SetSize( w,h );
*/
    dlg.ShowModal();

	RegenerateteCommandIds();

	m_RecreatePending = TRUE;
}

void wxPMBootstrapService::OnIdle( wxIdleEvent& event )
{
	if ( m_RecreatePending )
	{
		m_RecreatePending = FALSE;

		m_MenuItemsHash.Clear();

		RegenerateteCommandIds();

		m_KeyBinder.CreateMenuBar( m_pMainWnd );

		m_KeyBinder.UpdateKeyAssignments();
	}

	event.Skip(); // allow others do their tasks too
}

void wxPMBootstrapService::OnMenuEvent( wxEvent& event )
{
	DoTranslateEvent( event );
}

void wxPMBootstrapService::OnButtonEvent( wxEvent& event )
{
	DoTranslateEvent( event );
}

void wxPMBootstrapService::OnUpdateUIEvent( wxEvent& event )
{
	DoTranslateEvent( event );
}

int wxPMBootstrapService::RemoveServicesByClass( wxClassInfo* pSvcClass )
{
	size_t nFound = 0;
	PMServiceListT lst = m_Services;

	for( size_t i = 0; i != lst.Count(); ++i )
	
		if ( lst[i]->GetWxObject()->GetClassInfo() == pSvcClass )
		{
			StopService( lst[i] );
			RemoveService( lst[i] );
			++nFound;
		}

	return nFound;
}

bool wxPMBootstrapService::StartServicesByClass( wxClassInfo* pSvcClass )
{
	wxObject* pObj = pSvcClass->CreateObject();

	wxPMService* pSvc = (wxPMService*)pObj->GetRefData();

	AddService( pSvc );

	this->StartService( pSvc );

	return TRUE;
}

void wxPMBootstrapService::SetAppTitle( const wxString& appTitle )
{
	m_AppTitle = appTitle;
}

wxString& wxPMBootstrapService::GetAppTitle()
{
	return m_AppTitle;
}

void wxPMBootstrapService::ActiaveteRelatedPanes( wxPMDocument* pDoc, bool activate )
{
	wxString docType = pDoc->GetDocumentName();

	for( size_t i = 0; i != m_Services.Count(); ++i )
	{
		wxPMService* pSvc = m_Services[i];

		if ( pSvc->IsStarted() &&
			 pSvc->GetWxObject()->IsKindOf( CLASSINFO( wxPMPane ) ) )
		{
			wxPMPane* pPane = (wxPMPane*)pSvc->GetWxObject();

			wxStringList* pDocTypes = pPane->GetDocumentTypes();

			if ( pDocTypes )
			{
				for( size_t k = 0; k != pDocTypes->GetCount(); ++k )
				{
					// FIXME:: messy!

					if ( (*pDocTypes)[k] == docType )
					{
						if ( activate && !pPane->IsShown() )
						{
							ShowPane( pPane );
							break;
						}
						else
						if ( !activate && pPane->IsShown() )
						{
							HidePane( pPane );
							break;
						}

						if ( ( activate && pPane->IsShown() ) ||
							 ( !activate && !pPane->IsShown() ) )

							 break;
					}
				}

				if ( k == pDocTypes->GetCount() &&
					 pPane->IsShown() && activate )
				{
					 HidePane( pPane );
				}
			}
		}
	}
}
					 
/*** protected methods ***/

void wxPMBootstrapService::DoTranslateEvent( wxEvent& event )
{
	if ( m_RecreatePending ) return;

	wxCommandInfo* pItem = 
		(wxCommandInfo*)m_MenuItemsHash.Get( (long)event.GetId() );

	//wxASSERT_MSG( pItem != NULL, "wxCommandInfo object for the given event not found! not registered yet?" );

	if ( pItem == NULL )
	{
		event.Skip();
		return;
	}

	wxPMMenuItemData* pData = (wxPMMenuItemData*)pItem->GetUserData();

	// FIXME:: hack... how to pass the original id?? (needed in some cases)

	int id = event.GetId();

	//event.n = (char*)event.GetId();

	event.SetId( pData->m_ActualId ); // change id to that requested by it's owner

	if ( pData->m_pReceiver )

		pData->m_pReceiver->ProcessEvent( event );
}

wxCommandInfo* wxPMBootstrapService::FindItemByName( const wxString& path )
{
	return DoFindItemByName( path, m_KeyBinder.GetGroups() );
}

wxCommandInfo* wxPMBootstrapService::DoFindItemByName( const char* path, wxCommandInfoListT& lst )
{
	if ( path[0] == '/' ) ++path;

	if ( path[0] == '\0' ) return NULL;

	char* cur = (char*)(const char*)path;

	while( *cur && *cur != '/' ) ++cur;
	char tmp = *cur;

	*cur = '\0'; // terminate the first part temporarely

	for( size_t i = 0; i != lst.Count(); ++i )
	{
		if ( lst[i]->GetTitle() == path )
		{
			if ( tmp == '\0' )
			{
				*cur = tmp;
				return lst[i];
			}

			wxCommandInfo* pFound = NULL;

			if ( lst[i]->HasSubcommands() )
			
				// recurse to the next part of the menu-path
				pFound = DoFindItemByName( cur+1, *lst[i]->GetSubcommands() );

			if ( pFound )
			{
				*cur = tmp; // restore 

				return pFound;
			}
		}
	}

	*cur = tmp; // restore 

	return NULL;
}

void wxPMBootstrapService::DoMergeChain( wxPMMergableMenuChain* pChain,
									    wxEvtHandler* pReceiver )
{
	wxCommandInfo* pParent = NULL;
	wxCommandInfoListT* pLst = NULL;
	size_t insPos = 0;

	wxCommandInfo* pPred = FindItemByName( pChain->m_PredcessorPath );

	if ( pPred ) 
	{
		pParent = pPred->GetParent();
		pLst = pParent->GetSubcommands();
		insPos = (size_t)pLst->Index( pPred ); ++insPos; // after
	}
	else // insert at the end of the siblings list, if predcessor was not found
	{
		pLst = CreateTreePath( StripLastPathItem( pChain->m_PredcessorPath ), pParent );
		insPos = pLst->Count();
	}

	wxCommandInfo* pCommand = pChain->m_pCommands;

	while( pCommand->GetTitle() != "" )
	{

		wxPMMenuItemData* pPMData = new wxPMMenuItemData();
		pPMData->m_ActualId  = pCommand->GetId();
		pPMData->m_pReceiver = pReceiver;

		wxASSERT( pParent ); // DBG::

		wxCommandGroup* pNewItem = new wxCommandGroup();
		pCommand->Prepare();
		pCommand->AssignInfoTo( pNewItem );

		pNewItem->SetUserData( pPMData );
		pNewItem->SetCmdParent( pParent );

		pLst->Insert( pNewItem, insPos++ );

		++pCommand;
	}
}

void wxPMBootstrapService::DoRemoveChain( wxPMMergableMenuChain* pChain )
{
	wxCommandInfo* pCommand = pChain->m_pCommands;

	wxString path = StripLastPathItem( pChain->m_PredcessorPath );

	// remove matching items in one branch

	while( pCommand->GetTitle() != "" )
	{
		RemoveItemByPath( path + "/" + pCommand->GetTitle() );

		++pCommand;
	}

	// remove empty owned menu-path components ("those implicitely created by PMF")

	wxCommandInfo* pItem = FindItemByName( path );

	while( pItem && pItem->HasSubcommands() == FALSE )
	{
		wxCommandInfo* pParent = pItem->GetParent();

		if ( pParent )
            pItem->GetParent()->GetSubcommands()->Remove( pItem );
        else 
            m_KeyBinder.GetGroups().Remove( pItem );

		m_MenuItemsHash.Delete( (long)pItem->GetId() );
		delete pItem;

		pItem = pParent;
	}
}

void wxPMBootstrapService::MutateCommandToGroup( wxCommandInfoListT& lst, size_t cmdIdx )
{
	// our parent item is instance of wxCommandInfo, but
	// we need wxCommandInfoGroup for adding child-items,
	// do the mutation now

	wxCommandGroup* pGroup = new wxCommandGroup();
	wxCommandInfo* pCmd = lst[cmdIdx];

	pCmd->AssignInfoTo( pGroup );

	lst[cmdIdx] = pGroup;

	delete pCmd;
}

wxCommandInfoListT* wxPMBootstrapService::CreateTreePath( const wxString& path,
														  wxCommandInfo*& pParent )
{
	wxCommandInfoListT* pLst = &m_KeyBinder.GetGroups();

	size_t depth = 0; pParent = NULL;

	// search for existing components of given menu-path

	while( pLst )
	{
		wxString name = GetPathItem( path, depth ); bool found = FALSE;

		if ( name == "" ) break;

		for( size_t i = 0; i != pLst->Count(); ++i )
		{
			if ( (*pLst)[i]->GetTitle() == name )
			{
				/*
				if ( GetPathItem( path, depth+1 ) == "" &&
					 (*pLst)[i]->GetSubcommands() == NULL )
				{
					// if there are more path components required
					// after the current one, and the current one
					// is not a command-group (i.e. single command)
					// then convert it's type "on-the-fly"

					MutateCommandToGroup( *pLst, i );
				}
				*/

				pParent = (*pLst)[i];
				pLst = (*pLst)[i]->GetSubcommands(); found = TRUE;
				break;
			}
		}

		if ( !found )
		{
			// append missing path components

			while( name != "" )
			{
				wxPMMenuItemData* pPMData = new wxPMMenuItemData();
				pPMData->m_pReceiver = NULL;  // TBD:: really noone will receive it?
				pPMData->m_ActualId  = -1;

				wxCommandGroup* pCommand = 
					new wxCommandGroup( name );

				pCommand->SetId( 1033 );

				pCommand->SetUserData( pPMData );

				pLst->Add( pCommand );
				pCommand->SetCmdParent( pParent );
	
				// go deeper
				pLst    = pCommand->GetSubcommands();
				pParent = pCommand;

				name = GetPathItem( path, ++depth );
			}

			break;
		}
		else
			++depth;
	} 

	return pLst;
}

wxString wxPMBootstrapService::StripLastPathItem( const wxString& treePath )
{
	if ( treePath.Length() == 0 ) return treePath;

	size_t i = treePath.Length() - 1;

	while( i >= 0 && treePath[i] != '/' ) --i;

	if ( i == 1 || i == 0 ) return ""; // single "/" left as root, strip it

	return wxString( treePath, 0, i );
}

wxString wxPMBootstrapService::GetPathItem( const wxString& treePath, size_t i )
{
	size_t n = 0, len = treePath.Length();

	while( n < len )
	{
		if ( treePath[n] == '/' ) ++n;

		size_t start = n;

		while( n < len && treePath[n] != '/' ) ++n;

		if ( i == 0 )

			return wxString( treePath, start, n - start );

		++n; --i;
	}

	return "";
}

void wxPMBootstrapService::RemoveItemByPath( const wxString& treePath )
{
	wxCommandInfo* pItem = FindItemByName( treePath );

	if ( pItem == NULL ) return;

	if ( pItem->HasSubcommands() ) return; // cannot remove
										   // parent item when children are present

	if ( pItem->GetParent() ) pItem->GetParent()->GetSubcommands()->Remove( pItem );
						 else m_KeyBinder.GetGroups().Remove( pItem );

	m_MenuItemsHash.Delete( (long)pItem->GetId() );

	delete pItem;
}

void wxPMBootstrapService::DoRegenerateteCommandIds( wxCommandInfoListT& commands, int& nextId )
{
	for( size_t i = 0; i != commands.Count(); ++i )
	{
		wxCommandInfo& cmd = *commands[i];

		m_MenuItemsHash.Put( nextId, (wxObject*)&cmd );

		cmd.SetId( nextId++ );

		if ( cmd.HasSubcommands() )
		
			DoRegenerateteCommandIds( *cmd.GetSubcommands(), nextId );
	}
}

void wxPMBootstrapService::RegenerateteCommandIds()
{
	m_MenuItemsHash.Clear();

	int nextId = 500;

	DoRegenerateteCommandIds( m_KeyBinder.GetGroups(), nextId );
}

/* global access-point to the instance of bootstrap service */

static wxPMBootstrapService* _gBootstrapObj = NULL;

wxPMBootstrapService& wxGetPMBootstrapSvc()
{
	return *_gBootstrapObj;
}

void wxSetPMBootstrapSvc( wxPMBootstrapService* pSvc )
{
	_gBootstrapObj = pSvc;
}

wxPMBootstrapService& wxPMBootSvc()
{
	return *_gBootstrapObj;
}
