/////////////////////////////////////////////////////////////////////////////
// Name:        No names yet.
// Purpose:     Contrib. demo
// Author:      Aleksandras Gluchovas
// Modified by:
// Created:     2000/02/08
// RCS-ID:      
// Copyright:   (c) Aleksandras Gluchovas
// Licence:   	GPL (version 2.0 or later)
/////////////////////////////////////////////////////////////////////////////

#ifdef __GNUG__
    #pragma implementation "pm_flmgr.cpp"
    #pragma interface "pm_flmgr.cpp"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include "wx/pmf/pm_flmgr.h"

// FL-stuff
#include "wx/fl/controlbar.h"
#include "wx/fl/barhintspl.h"
#include "wx/fl/cbcustom.h"
#include "wx/fl/updatesmgr.h"

#include "wx/pmf/pm.h" // import wxPMBootStrapService

/***** Implementation for class wxPMFrameLayoutPreferenceTemplate *****/

wxPMFrameLayoutPreferenceTemplate::wxPMFrameLayoutPreferenceTemplate( 
									wxClassInfo* pPaneClass,
								    wxPMFrameLayoutPreference* pPreference )

	: m_pPaneClass( pPaneClass ),
	  m_pPrefrence( pPreference )
{}

wxPMFrameLayoutPreferenceTemplate::~wxPMFrameLayoutPreferenceTemplate()
{
	if ( m_pPrefrence ) delete m_pPrefrence;
}

/***** Implementation for class wxPMFrameLayoutAlgorithm *****/

IMPLEMENT_DYNAMIC_CLASS( wxPMFrameLayoutAlgorithm, wxPMLayoutAlgorithm )

wxPMFrameLayoutAlgorithm::wxPMFrameLayoutAlgorithm()

	: m_RelayoutPending( FALSE ),
	  m_NextWndId( 9000 )
{
}

wxPMFrameLayoutAlgorithm::~wxPMFrameLayoutAlgorithm()
{
	WX_CLEAR_ARRAY( m_PrefTemplates );
}

bool wxPMFrameLayoutAlgorithm::Start()
{
    m_pLayout = new wxFrameLayout( wxPMBootSvc().GetMainWindow(), 
								  wxPMBootSvc().GetMainWindow()->GetClientWindow() );

	m_pLayout->EnableFloating( FALSE );

	m_pLayout->PushDefaultPlugins();
	m_pLayout->AddPlugin( CLASSINFO( cbBarHintsPlugin ) ); // fancy "X"es and bevel for bars
	m_pLayout->AddPlugin( CLASSINFO( cbBarHintsPlugin ) ); // fancy "X"es and bevel for bars
	m_pLayout->AddPlugin( CLASSINFO( cbSimpleCustomizationPlugin ) ); // popups for showing/hiding bars

	m_pLayout->SetUpdatesManager( new cbSimpleUpdatesMgr( m_pLayout ) );
	
	cbCommonPaneProperties props;
	m_pLayout->GetPaneProperties( props );
	props.mRealTimeUpdatesOn = FALSE;
	m_pLayout->SetPaneProperties( props );
	
	wxPMBootSvc().AddListener( this );

	return TRUE;
}

bool wxPMFrameLayoutAlgorithm::Stop()
{
	// FIXME:: ensure ordering...

	wxPMBootSvc().RemoveListener( this );

	if ( m_pLayout ) 
        delete m_pLayout;

	return TRUE;
}

void wxPMFrameLayoutAlgorithm::OnPaneShown( wxPMPane& pane )
{
	if ( pane.GetParentPane() == NULL )
	{
		//m_ActivePanes.Add( &pane );

		if ( pane.GetPaneWindow() == NULL )
		{

			cbDimInfo sizes( 240,220, // when docked horizontally      
							 240,220,    // when docked vertically        
							 200,150, // when floated                  
							 TRUE,  // the bar is fixed-size
							 5,     // vertical gap (bar border)
							 5      // horizontal gap (bar border)
						   ); 

			sizes.mIsFixed = FALSE;

			wxWindow* pMainWnd = wxPMBootSvc().GetMainWindow();

			pane.CreatePaneWindow( pMainWnd );

			m_pLayout->AddBar( pane.GetPaneWindow(),// bar window
							  sizes, wxLEFT,// alignment ( 0-top,1-bottom, etc)
							  0,                   // insert into 0th row (vert. position)
							  0,                   // offset from the start of row (in pixels)
							  "Pane's name TBD::"  // name to refere in customization pop-ups
						   );

			// TBD:: regard wxPMFrameLayoutPreference for the given pane

			m_pLayout->RecalcLayout( TRUE );
		}
		else
		{
			cbBarInfo* pBar = m_pLayout->FindBarByWindow( pane.GetPaneWindow() );

			if ( pBar->mState == wxCBAR_HIDDEN )

				m_pLayout->InverseVisibility( pBar );
		}

	//	pane.GetPaneWindow()->Show( TRUE );
    
	//	m_RelayoutPending = TRUE;
	}
}

void wxPMFrameLayoutAlgorithm::OnPaneHidden( wxPMPane& pane )
{
	if ( pane.GetParentPane() == NULL )
	{
		//m_ActivePanes.Remove( &pane );

		cbBarInfo* pBar = m_pLayout->FindBarByWindow( pane.GetPaneWindow() );

		if ( pBar->mState != wxCBAR_HIDDEN )

			m_pLayout->InverseVisibility( pBar );

	//	m_RelayoutPending = TRUE;
	}
}

void wxPMFrameLayoutAlgorithm::OnServiceStarted( wxPMService& service )
{
	/*
	if ( service.GetWxObject()->IsKindOf( CLASSINFO( wxPMPane ) ) )
	{
		wxPMPane* pPane = (wxPMPane*)service.GetWxObject();

		if ( pPane->GetParentPane() == NULL )
		
			m_TopLevelPanes.Add( pPane );
	}
	*/
}

void wxPMFrameLayoutAlgorithm::OnServiceStopped( wxPMService& service )
{
	/*
	if ( service.GetWxObject()->IsKindOf( CLASSINFO( wxPMPane ) ) )
	{
		wxPMPane* pPane = (wxPMPane*)service.GetWxObject();

		if ( pPane->GetParentPane() == NULL )
		
			m_TopLevelPanes.Remove( pPane );
	}
	*/
}


wxPMLayoutPreference* wxPMFrameLayoutAlgorithm::CreateDefaultLayoutPrefs()
{
	return NULL;
}

void wxPMFrameLayoutAlgorithm::Serialize( wxConfigBase& cfg, const wxString& key, bool isLoading )
{
}

void wxPMFrameLayoutAlgorithm::AddPrefernceTemplate( wxPMFrameLayoutPreferenceTemplate* pTempl )
{
	m_PrefTemplates.Add( pTempl );
}

void wxPMFrameLayoutAlgorithm::AddPrefernceTemplate( wxClassInfo* pClass, 
												    wxPMFrameLayoutPreference* pPref )
{
	m_PrefTemplates.Add( new wxPMFrameLayoutPreferenceTemplate( pClass, pPref ) );
}

