/////////////////////////////////////////////////////////////////////////////
// Name:        No names yet.
// Purpose:     
// Author:      Aleksandras Gluchovas
// Modified by:
// Created:     2000/02/08
// RCS-ID:      
// Copyright:   (c) Aleksandras Gluchovas
// Licence:   	GPL (version 2.0 or later)
/////////////////////////////////////////////////////////////////////////////

#ifdef __GNUG__
    #pragma implementation "res_doc.cpp"
    #pragma interface "res_doc.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include "res_doc.h"
#include "menuedit.h"
#include "widgeteditor.h" // FIXME:: HACK:: should not be coupled!

// sorry, 3 .cpp's and .h's of AWT-layouts contrib. needed to compile this .cpp!

#include "wx/awtlay/awt_emul.h"
#include "wx/awtlay/borderlayout.h"
#include "wx/awtlay/flowlayout.h"

#include "wx/choicdlg.h"
#include "wx/textdlg.h"


/***** Implementation for class ResourceDocument *****/

wxDMClipboard* ResDocument::mpClipboard = NULL;
ResTemplateManager* ResDocument::mpGlobalMgr = NULL;

IMPLEMENT_DYNAMIC_CLASS( ResDocument, wxPMDocument )

ResDocument::ResDocument()

	: mpRoot( NULL ),
	  mpCurRec( NULL ),
	  mDocsHash( wxKEY_STRING )
{}

ResDocument::~ResDocument()
{
	if ( mpRoot ) delete mpRoot;
}

ResTemplateManager& ResDocument::GetTemplateManager()
{
	if ( mpGlobalMgr == NULL )
	{
		mpGlobalMgr = new ResTemplateManager();

		bool ok = mpGlobalMgr->LoadResourceTemplates( "config_widgets.txt" );
	}

	return *mpGlobalMgr;
}

void ResDocument::GetTopLevelRecordsByType( const wxString& type, FieldListT& toLst )
{
	if ( !mpRoot ) return;

	for( size_t i = 0; i != mpRoot->GetFieldCount(); ++i )

		if ( mpRoot->GetFieldByIndex(i).GetName() == type )

			toLst.Add( &mpRoot->GetFieldByIndex(i) );
}

bool ResDocument::CanHaveFile()
{
	return TRUE;
}

bool ResDocument::SavePMDocument( wxOutputStream& s )
{
	if ( !mpRoot ) return NULL;

	//wxString txt = mpRoot->GetTextContent();
	wxString txt = mpRoot->GetXMLTextContent();

	s.Write( (const char*)txt, txt.Length() );

	return TRUE;
}

bool ResDocument::LoadPMDocument( wxInputStream& s )
{
	wxString script;
	bool ok = wxPMReadStreamToString( s, script );

	if ( !ok ) return FALSE;

	if ( mpRoot ) delete mpRoot;

	mpRoot = NULL;

	InMemoryXMLRecordSet rset;
	rset.Open( script );

	rset.MoveFirst();

	while( !rset.IsEof() )
	{
		if ( rset.GetRecord().GetName() == "resource" )
		{
			mpRoot = (Record*)rset.GetRecord().Clone();
			break;
		}

		rset.MoveNext();
	}

	return mpRoot != NULL;
}

void ResDocument::OnActivatePMDocument()
{
	//wxMessageBox( "ResDocument::OnActivatePMDocument() called!" );

	ResSelectionDialog dlg( wxPMBootSvc().GetMainWindow(), this );

	dlg.ShowModal();
}

void ResDocument::OnClosingChildDocument( wxPMDocument* pChild, bool wasSaved )
{
	if ( pChild->IsModified() )
		Modify( this != NULL );

	Record* pRec = (Record*)mRecsHash.Get( (long)pChild );

	mRecsHash.Delete( (long)pChild );
	mDocsHash.Delete( (*pRec)[ "name" ] );
}

bool ResDocument::OnCreate(const wxString& path, long flags)
{
	GetTemplateManager(); // make sure it's loaded

	if ( mpRoot ) delete mpRoot;

	mpRoot = new Record();

	mpRoot->SetName( "resource" );

	return TRUE;
}

wxString ResDocument::GetItemClassByTag( const wxString& itemType )
{
	if ( itemType == "menubar" )

		return "wxMenuBar";
	else
	if ( itemType == "menu" || itemType == "item" )

		return "wxMenuItem";

	WidgetResTemplate* pTempl = mpGlobalMgr->FindWidgetTemplateForTag( itemType );

	if ( pTempl )

		return pTempl->mWidgetClass;
	else
		return "unknown_widget_class";
}

wxString ResDocument::GetItemClassById( const wxString& itemId )
{
	for( size_t i = 0; i != mpRoot->GetFieldCount(); ++i )
	{
		if ( mpRoot->GetRecordByIndex(i)[ "name" ] == itemId )

			return GetItemClassByTag( mpRoot->GetRecordByIndex(i).GetName() );
	}

	return "unknown_widget_class";
}

void ResDocument::GetSubItemIds( const wxString& parentItem,
								 wxStringList& ids,
								 wxStringList& classes )
{
	wxPMDocument* pDoc = (wxPMDocument*)mDocsHash.Get( parentItem );

	if ( pDoc )
	{
		wxDMNode* pRoot = NULL;

		// FIXME:: hacks...

		if ( pDoc->IsKindOf( CLASSINFO( MenuEditorDoc ) ) )
		{
			MenuEditorDoc* pMenuDoc = (MenuEditorDoc*)pDoc;

			pRoot = pMenuDoc->GetRootNode();
		}
		else
		if ( pDoc->IsKindOf( CLASSINFO( WidgetDocument ) ) )
		{
			WidgetDocument* pWidgetDoc = (WidgetDocument*)pDoc;

			pRoot = pWidgetDoc->GetRootNode();
		}
		else
			return;

		for( size_t i = 0; i != pRoot->GetChildCount(); ++i )
		
			GetSubItemIdsRecursive( *pRoot->GetChild(i), ids, classes );
	}
	else
	{
		Record* pRec = FindItemByName( parentItem );

		if ( !pRec ) return;

		for( size_t i = 0; i != pRec->GetFieldCount(); ++i )
		
			if ( pRec->GetFieldByIndex(i).GetType() == FT_RECORD )

				GetSubItemIdsRecursive( pRec->GetRecordByIndex(i), ids, classes );
	}
}

void ResDocument::GetSubItemIdsRecursive( Record& rec,
										  wxStringList& ids,
										  wxStringList& classes )
{
	if ( rec.FindField( "name" ) )
	{
		if ( rec.GetName() != "separator" )
		{
			ids.Add( rec[ "name" ] );

			classes.Add( GetItemClassByTag( rec.GetName() ) );
		}
	}

	for( size_t i = 0; i != rec.GetFieldCount(); ++i )

		if ( rec.GetFieldByIndex(i).GetType() == FT_RECORD )

			GetSubItemIdsRecursive( rec.GetRecordByIndex(i), ids, classes );
}

void ResDocument::GetSubItemIdsRecursive( wxDMNode& node,
										  wxStringList& ids,
										  wxStringList& classes )
{
	// FIXME:: HACK:: should be tag-based class identification

	wxDMAttrGroupInfo* pGroup = node.GetAttrGroupInfo(0);
	
	wxString id = 
		
		pGroup->GetAttrByName("name")->GetStringValue( node.GetAttrGroupData((size_t)0) );


	if ( node.IsKindOf( CLASSINFO( wxME_Node ) ) )
	{
		wxString isSepartor = 
		
			pGroup->GetAttrByName("separator")->GetStringValue( node.GetAttrGroupData((size_t)0) );

		if ( isSepartor != "true" )
		{
			classes.Add( "wxMenuItem" );
			ids.Add( id );
		}
	}
	else
	if ( node.IsKindOf( CLASSINFO( WidgetNode ) ) )
	{
		classes.Add( ((WidgetNode*)&node)->mpTempl->mWidgetClass );
		ids.Add( id );
	}
	else
	{
		classes.Add( "wx__Uknown__Widget" );
		ids.Add( id );
	}

	for( size_t i = 0; i != node.GetChildCount(); ++i )

		GetSubItemIdsRecursive( *node.GetChild(i), ids, classes );
}

Record* ResDocument::GetRootRecord()
{
	return mpRoot;
}

Record* ResDocument::GetCurrentRecord()
{
	return mpCurRec;
}

bool ResDocument::IsActiaveted( const wxString& itemName )
{
	return mDocsHash.Get( itemName ) != NULL;
}

void ResDocument::RemoveItemByName( const wxString& itemName )
{
	Record* pItem = FindItemByName( itemName );

	if ( pItem )

		mpRoot->RemoveField( pItem );

	Modify( TRUE );
}

Record* ResDocument::FindItemByName( const wxString& itemName )
{
	for( size_t i = 0; i != mpRoot->GetFieldCount(); ++i )
	{
		Record& rec = mpRoot->GetRecordByIndex(i);

		if ( rec.FindField( "name" ) && rec[ "name" ] == itemName )
		
			return &rec;
	}

	return NULL;
}

ResItemDocumentBase* ResDocument::ActivateEditorForItem( const wxString& itemName )
{
	wxPMDocument* pDoc = (wxPMDocument*)mDocsHash.Get( itemName );

	wxPMDocManager& mgr = *wxPMBootSvc().GetDocManager();

	if ( pDoc )
	{
		mgr.ActivateDocument( pDoc );

		return (ResItemDocumentBase*)pDoc;
	}

	Record* pRec = FindItemByName( itemName );

	if ( pRec )
	{
		wxDocTemplate* pTempl = NULL;
		wxPMDocument* pDoc = NULL;
		mpCurRec = pRec;

		if ( pRec->GetName() == "menubar" )

			pTempl = mgr.FindTemplateForDocType( "text/x-menubar-resource" );
		else
			pTempl = mgr.FindTemplateForDocType( "text/x-widget-resource" );

		if ( pTempl )
		{
			pDoc = mgr.CreatePMDocument( wxPMFileInfo(), wxDOC_SILENT, pTempl, this );

			wxPMView* pView = (wxPMView*)pDoc->GetFirstView();

			pView->GetViewFrame()->SetTitle( itemName );
			pDoc->SetTitle( itemName );

			mDocsHash.Put( itemName, pDoc );
			mRecsHash.Put( (long)pDoc, (wxObject*)pRec );
		}

		return (ResItemDocumentBase*)pDoc;
	}

//glt - No return value in this situation!!!
}

wxDMClipboard& ResDocument::GetClipboard()
{
	if ( !mpClipboard )

		mpClipboard = new wxDMClipboard();

	return *mpClipboard;
}

/***** Implementation for class ResSelectionDialog *****/

BEGIN_EVENT_TABLE( ResSelectionDialog, wxDialog )

	EVT_BUTTON( wxID_OK,   ResSelectionDialog::OnOk )
	EVT_BUTTON( ID_NEW,    ResSelectionDialog::OnNew )
	EVT_BUTTON( ID_REMOVE, ResSelectionDialog::OnRemove )

	EVT_LISTBOX( ID_LIST_BOX, ResSelectionDialog::OnSelected )
	EVT_LISTBOX_DCLICK( ID_LIST_BOX, ResSelectionDialog::OnDblClick )

END_EVENT_TABLE()

ResSelectionDialog::ResSelectionDialog( wxWindow* parent, ResDocument* pDoc )

	: wxDialog( parent, -1, "Select resource", wxDefaultPosition, wxSize( 300, 250 ),
			    wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER ),
	  mpDoc( pDoc )
{
	Container& cont = *new Container( this );
	cont.setInsets( Insets(10,10,10,10) );
	cont.setLayout( new BorderLayout( 10, 10 ) );

	mpBox = new wxListBox( this, ID_LIST_BOX );
	mpOkBtn = new wxButton( this, wxID_OK, "&Edit", wxDefaultPosition );
	mpNewBtn = new wxButton( this, ID_NEW, "&New", wxDefaultPosition );
	mpRemoveBtn = new wxButton( this, ID_REMOVE, "&Remove", wxDefaultPosition );
	mpCancelBtn = new wxButton( this, wxID_CANCEL, "&Cancel", wxDefaultPosition );

	Container* btnPanel = new Container();
	btnPanel->setInsets( Insets(5,5,5,5) );
	btnPanel->setLayout( new FlowLayout() ); 

	btnPanel->add( mpOkBtn );
	btnPanel->add( mpNewBtn );
	btnPanel->add( mpRemoveBtn );
	btnPanel->add( mpCancelBtn );

	cont.add( mpBox,	BorderLayout::CENTER );
	cont.add( btnPanel, BorderLayout::SOUTH );

	mpOkBtn->SetDefault();

	FillListBox();

	cont.doLayout();
	Center( wxBOTH );

	mpBox->SetFocus();

	wxCommandEvent dummy;
	OnSelected( dummy ); // enable/disable buttons
}

void ResSelectionDialog::FillListBox()
{
	Record& rec = *mpDoc->GetRootRecord();

	for( size_t i = 0; i != rec.GetFieldCount(); ++i )
	{
		wxString name = rec.GetRecordByIndex( i )[ "name" ];

		mpBox->Append( name );
	}

	if ( mpBox->Number() )

		mpBox->SetSelection( 0, TRUE );
}

void ResSelectionDialog::OnOk( wxCommandEvent& event )
{
	wxString name = mpBox->GetStringSelection();

	EndModal( wxID_OK );

	mpDoc->ActivateEditorForItem( name );
}

void ResSelectionDialog::OnNew( wxCommandEvent& event )
{
	WidgetResTemplateListT& lst = mpDoc->GetTemplateManager().GetTemplates();

	wxString* choices = new wxString[ lst.Count() + 1 ];

	choices[ 0 ] = "menubar";

	for( size_t i = 0; i != lst.Count(); ++i )
	
		choices[i+1] = lst[i]->mTag;

	wxSingleChoiceDialog dlg( wxPMBootSvc().GetMainWindow(), "Select item to craete", 
						      "New Item", lst.Count(), choices );

	if ( dlg.ShowModal() == wxID_OK )
	{
		wxString tag = dlg.GetStringSelection();

		wxTextEntryDialog dlg1( wxPMBootSvc().GetMainWindow(), 
							    "Enter name for the new item:", "Item Name",
							    tag + "1" );

		// TBD:: check for name-collisions

		if ( dlg1.ShowModal() == wxID_OK )
		{
			EndModal( wxID_OK );

			Record* pItemRec = new Record();

			pItemRec->SetName( tag );
			wxString name = dlg1.GetValue();

			pItemRec->AddStringField( "name", name );

			mpDoc->GetRootRecord()->AddField( pItemRec );

			if ( tag == "menubar" )
			{
				Record* pFirstItem = new Record();
				pFirstItem->SetName( "menu" );
				pFirstItem->AddStringField( "name", "menu1" ); 
				pFirstItem->SetStringValue( "First Menu" );

				pItemRec->AddField( pFirstItem );
			}
			else
			{
				pItemRec->AddStringField( "pos", "0,0" ); 
				pItemRec->AddStringField( "size", "300,200" ); 
			}

			mpDoc->Modify( TRUE );

			mpDoc->ActivateEditorForItem( name );
		}
	}
}

void ResSelectionDialog::OnRemove( wxCommandEvent& event )
{
	if ( mpBox->GetSelection() == -1 )
	{
		wxMessageBox( "Please, select item first" );

		return;
	}

	if ( mpDoc->IsActiaveted( mpBox->GetStringSelection() ) )

		return;

	mpDoc->RemoveItemByName( mpBox->GetStringSelection() );

	mpBox->Clear();

	FillListBox();

	if ( mpBox->Number() == 0 )

		mpRemoveBtn->Enable( FALSE );
}

void ResSelectionDialog::OnSelected( wxCommandEvent& event )
{
	if ( mpDoc->IsActiaveted( mpBox->GetStringSelection() ) )

		mpRemoveBtn->Enable( FALSE );
	else
		mpRemoveBtn->Enable( TRUE );
}

void ResSelectionDialog::OnDblClick( wxCommandEvent& event )
{
	OnOk( event );
}

/***** Implementation for class ResItemDocumentBase *****/

IMPLEMENT_ABSTRACT_CLASS( ResItemDocumentBase, wxPMDocument )
