/////////////////////////////////////////////////////////////////////////////
// Name:        No names yet.
// Purpose:     Contrib. demo
// Author:      Aleksandras Gluchovas
// Modified by:
// Created:     2000/02/08
// RCS-ID:      
// Copyright:   (c) Aleksandras Gluchovas
// Licence:   	GPL (version 2.0 or later)
/////////////////////////////////////////////////////////////////////////////

#ifdef __GNUG__
    #pragma implementation "minimal.cpp"
    #pragma interface "minimal.cpp"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include "res_templ.h"
#include "ui_lookrenderer.h"

#include <stdlib.h> // atoi(..)
#include "myutils.h"

#include "wx/colordlg.h"

/***** Implementation for class ResGroupData *****/

ResGroupData::ResGroupData()

	: mpGroupInfo( NULL )
{}

ResGroupData::~ResGroupData()
{
	if ( mpGroupInfo )
	{
		mpGroupInfo->DestroyValues( GetData() );

		mpGroupInfo = NULL;
	}
}

void ResGroupData::SetGroupInfo( wxDMAttrGroupInfo* pInfo )
{
	if ( mpGroupInfo ) 
		
		CleanUp();

	// data-array is prepared immediately after setting meta-info

	mpGroupInfo = pInfo;

    size_t i;
	for( i = 0; i != pInfo->Count(); ++i )

		mAttrData.Add( NULL );
	
	mpGroupInfo->InitValues( GetData() );
}

wxDMAttrGroupInfo* ResGroupData::GetGroupInfo()
{
	return mpGroupInfo;
}


void* ResGroupData::GetData()
{
	return (void*)this;
}

void ResGroupData::CopyDataTo( ResGroupData& other )
{
	if ( mpGroupInfo == NULL )
	
		other.CleanUp();
	else
	{
		other.SetGroupInfo( mpGroupInfo );

		mpGroupInfo->CopyValues( other.GetData(), this->GetData() );
	}
}

/*** protected methods ***/

void ResGroupData::CleanUp()
{
	if ( !mpGroupInfo ) return;

	mpGroupInfo->DestroyValues( GetData() );

	mAttrData.Clear();

	mpGroupInfo = NULL;
}

/***** Implementation for class ResIntAttrInfo *****/

wxString ResIntAttrInfo::GetStringValue( void* pGroupData )
{
	int i = (int)
		((ResGroupData*)pGroupData)->mAttrData[ (size_t)GetUserData() ];

	return IntToStr( i );
}

void ResIntAttrInfo::SetStringValue( const wxString& value, void* pGroupData )
{
	int& i = (int&)
		((ResGroupData*)pGroupData)->mAttrData[ (size_t)GetUserData() ];

	i = atoi( value.c_str() );
}

bool ResIntAttrInfo::ValidateStringValue( const wxString& value, const wxString* pErrMsg )
{
	return TRUE; // TBD:: checks
}

void ResIntAttrInfo::InitValue( void* pGroupData )
{
	int& i = (int&)
		((ResGroupData*)pGroupData)->mAttrData[ (size_t)GetUserData() ];

	i = atoi( mDefaultValue.c_str() );
}

void ResIntAttrInfo::CopyValue( void* pDestData, void* pSrcData )
{
	int src = (int&)
		((ResGroupData*)pSrcData)->mAttrData[ (size_t)GetUserData() ];

	int& dest = (int&)
		((ResGroupData*)pDestData)->mAttrData[ (size_t)GetUserData() ];

	dest = src;
}

/***** Implementation for class ResBoolAttrInfo *****/

wxString ResBoolAttrInfo::GetStringValue( void* pGroupData )
{
	bool b = 
		((ResGroupData*)pGroupData)->mAttrData[ (size_t)GetUserData() ] != 0;

	return b ? "true" : "false" ;
}

void ResBoolAttrInfo::SetStringValue( const wxString& value, void* pGroupData )
{
	bool& b = (bool&)
		((ResGroupData*)pGroupData)->mAttrData[ (size_t)GetUserData() ];

	b = value == "true";
}

bool ResBoolAttrInfo::ValidateStringValue( const wxString& value, const wxString* pErrMsg )
{
	return TRUE; // TBD:: checks
}

void ResBoolAttrInfo::InitValue( void* pGroupData )
{
	bool& b = (bool&)
		((ResGroupData*)pGroupData)->mAttrData[ (size_t)GetUserData() ];

	b = mDefaultValue == "true";
}

void ResBoolAttrInfo::CopyValue( void* pDestData, void* pSrcData )
{
	bool src = (bool&)
		((ResGroupData*)pSrcData)->mAttrData[ (size_t)GetUserData() ];

	bool& dest = (bool&)
		((ResGroupData*)pDestData)->mAttrData[ (size_t)GetUserData() ];

	dest = src;
}

bool ResBoolAttrInfo::ShownAsBooleanField()
{
	return TRUE;
}

/***** Implementation for class ResStringAttrInfo *****/

wxString ResStringAttrInfo::GetStringValue( void* pGroupData )
{
	wxString* pStr = (wxString*)
		((ResGroupData*)pGroupData)->mAttrData[ (size_t)GetUserData() ];

	return *pStr;
}

void ResStringAttrInfo::SetStringValue( const wxString& value, void* pGroupData )
{
	wxString* pStr = (wxString*)
		((ResGroupData*)pGroupData)->mAttrData[ (size_t)GetUserData() ];

	*pStr = value;
}

bool ResStringAttrInfo::ValidateStringValue( const wxString& value, const wxString* pErrMsg )
{
	return TRUE; // TBD:: checks
}

void ResStringAttrInfo::InitValue( void* pGroupData )
{
	wxString*& pStr = (wxString*&)
		((ResGroupData*)pGroupData)->mAttrData[ (size_t)GetUserData() ];

	pStr = new wxString( mDefaultValue );
}

void ResStringAttrInfo::DestroyValue( void* pGroupData )
{
	wxString* pStr = (wxString*)
		((ResGroupData*)pGroupData)->mAttrData[ (size_t)GetUserData() ];

	delete pStr;
}

void ResStringAttrInfo::CopyValue( void* pDestData, void* pSrcData )
{
	wxString* pSrcStr = (wxString*&)
		((ResGroupData*)pSrcData)->mAttrData[ (size_t)GetUserData() ];

	wxString*& pDestStr = (wxString*&)
		((ResGroupData*)pDestData)->mAttrData[ (size_t)GetUserData() ];

	*pDestStr = *pSrcStr;
}

/***** Implementation for class ResColorAttrInfo *****/

wxString ResColorAttrInfo::GetStringValue( void* pGroupData )
{
	int rgb = (int)
		((ResGroupData*)pGroupData)->mAttrData[ (size_t)GetUserData() ];

	return RGBIntToStr( rgb );
}

void ResColorAttrInfo::SetStringValue( const wxString& value, void* pGroupData )
{
	int& rgb = (int&)
		((ResGroupData*)pGroupData)->mAttrData[ (size_t)GetUserData() ];

	rgb = StrToRGbInt( value );
}

bool ResColorAttrInfo::ValidateStringValue( const wxString& value, const wxString* pErrMsg )
{
	return TRUE; // TBD::
}

bool ResColorAttrInfo::HasValueSelectionDialog()
{
	return TRUE;
}

bool ResColorAttrInfo::ShowValueSelectonDialog( void* pGroupData )
{
	int& rgb = (int&)
		((ResGroupData*)pGroupData)->mAttrData[ (size_t)GetUserData() ];

	wxColor c( ::GetIntRedValue(rgb), ::GetIntGreenValue(rgb), ::GetIntBlueValue(rgb) );
	wxColourData cdata;	cdata.SetColour( c );

	wxColourDialog dlg( NULL, &cdata );

	if ( dlg.ShowModal() == wxID_OK )
	{
		c = cdata.GetColour();

		rgb = MakeRGBInt( c.Red(), c.Green(), c.Blue() );

		return TRUE;
	}
	else
		return FALSE;
}

void ResColorAttrInfo::InitValue( void* pGroupData )
{
	SetStringValue( mDefaultValue, pGroupData );
}

void ResColorAttrInfo::CopyValue( void* pDestData, void* pSrcData )
{
	int srcRgb = (int&)
		((ResGroupData*)pSrcData)->mAttrData[ (size_t)GetUserData() ];

	int& destRgb = (int&)
		((ResGroupData*)pDestData)->mAttrData[ (size_t)GetUserData() ];

	destRgb = srcRgb;
}

/***** Implementation for class ResRestrictedStringAttrInfo *****/

ResRestrictedStringAttrInfo::ResRestrictedStringAttrInfo()

	: mShownAsBool( FALSE )
{}

wxString ResRestrictedStringAttrInfo::GetStringValue( void* pGroupData )
{
	int index = (int)
		((ResGroupData*)pGroupData)->mAttrData[ (size_t)GetUserData() ];

	return mValues[index];
}

void ResRestrictedStringAttrInfo::SetStringValue( const wxString& value, void* pGroupData )
{
	int& index = (int&)
		((ResGroupData*)pGroupData)->mAttrData[ (size_t)GetUserData() ];

    size_t i;
	for( i = 0; i != mValues.GetCount(); ++i )

		if ( mValues[i] == value )
		{
			index = (int)i;
			break;
		}
}

bool ResRestrictedStringAttrInfo::ValidateStringValue( const wxString& value, const wxString* pErrMsg )
{
	return TRUE; // TBD:: check
}

bool ResRestrictedStringAttrInfo::HasRestrictedValues( void* pGroupData )
{
	return TRUE;
}

bool ResRestrictedStringAttrInfo::SetRestrictedValues( wxStringList& list )
{
	mValues = list;

	return TRUE;
}

void ResRestrictedStringAttrInfo::GetRestrictedValues( wxStringList& toLst, void* pGroupData )
{
	toLst = mValues;
}

void ResRestrictedStringAttrInfo::InitValue( void* pGroupData )
{
	SetStringValue( mDefaultValue, pGroupData );
}

void ResRestrictedStringAttrInfo::CopyValue( void* pDestData, void* pSrcData )
{
	int srcIndex = (int&)
		((ResGroupData*)pSrcData)->mAttrData[ (size_t)GetUserData() ];

	int& destIndex = (int&)
		((ResGroupData*)pDestData)->mAttrData[ (size_t)GetUserData() ];

	destIndex = srcIndex;
}

bool ResRestrictedStringAttrInfo::ShownAsBooleanField()
{
	return mShownAsBool;
}

void ResRestrictedStringAttrInfo::SetShownAsBoolean( bool on )
{
	mShownAsBool = on;
}

/***** Implementation for class ResStringListAttrInfo *****/

wxString ResStringListAttrInfo::GetStringValue( void* pGroupData )
{
	return "";
}

void ResStringListAttrInfo::SetStringValue( const wxString& value, void* pGroupData )
{
	// nothing
}

bool ResStringListAttrInfo::ValidateStringValue( const wxString& value, const wxString* pErrMsg )
{
	return TRUE;
}

bool ResStringListAttrInfo::HasMultipleValues( void* pGroupData )
{
	return TRUE;
}

void ResStringListAttrInfo::SetMultipleValues( wxStringList& fromList, void* pGroupData )
{
	wxStringList* pLst = (wxStringList*)
		((ResGroupData*)pGroupData)->mAttrData[ (size_t)GetUserData() ];

	*pLst = fromList;
}

void ResStringListAttrInfo::GetMultipleValues( wxStringList& toList, void* pGroupData )
{
	wxStringList* pLst = (wxStringList*)
		((ResGroupData*)pGroupData)->mAttrData[ (size_t)GetUserData() ];

	toList = *pLst;
}

void ResStringListAttrInfo::InitValue( void* pGroupData )
{
	wxStringList*& pLst = (wxStringList*&)
		((ResGroupData*)pGroupData)->mAttrData[ (size_t)GetUserData() ];

	pLst = new wxStringList();
}

void ResStringListAttrInfo::DestroyValue( void* pGroupData )
{
	wxStringList* pLst = (wxStringList*)
		((ResGroupData*)pGroupData)->mAttrData[ (size_t)GetUserData() ];

	delete pLst;
}

void ResStringListAttrInfo::CopyValue( void* pDestData, void* pSrcData )
{
	wxStringList* pSrcLst = (wxStringList*&)
		((ResGroupData*)pSrcData)->mAttrData[ (size_t)GetUserData() ];

	wxStringList*& pDestLst = (wxStringList*&)
		((ResGroupData*)pDestData)->mAttrData[ (size_t)GetUserData() ];

	*pDestLst = *pSrcLst;
}

/***** Implementation for class ResPropInfo *****/

ResPropInfo::ResPropInfo()

	: mType(-1),
	  mVisibileOnly( FALSE )
{}

ResPropInfo::~ResPropInfo()
{}

ResPropSetInfo::~ResPropSetInfo()
{
    size_t i;
	for( i = 0; i != mProps.Count(); ++i )

		if ( mProps[i]->mType != RESPROP_PROPSET )
		
			delete mProps[i];
}

/***** Implementation for class ResPropInfo *****/

bool ResPropInfo::HasRestrictedValue( const wxString& value )
{
    size_t i;
	for( i = 0; i != mRestrValues.GetCount(); ++i )

		if ( mRestrValues[i] == value )

			return TRUE;

	return FALSE;
}

wxString ResPropInfo::TranslateToDispValue( const wxString& value )
{
    size_t i;
	for( i = 0; i != mRestrValues.GetCount(); ++i )

		if ( mRestrValues[i] == value )

			return mRestrDispValues[i];

	return "";
}

wxString ResPropInfo::TranslateFromDispValue( const wxString& dispValue )
{
    size_t i;
	for( i = 0; i != mRestrDispValues.GetCount(); ++i )

		if ( mRestrDispValues[i] == dispValue )

			return mRestrValues[i];

	return "";
}

/***** Implementation for class WidgetResTemplate *****/

WidgetResTemplate::~WidgetResTemplate()
{
}

/***** Implementation for class ResTemplateManager *****/

ResTemplateManager::ResTemplateManager()

	: mTagHash( wxKEY_STRING )
{
}

ResTemplateManager::~ResTemplateManager()
{
	WX_CLEAR_ARRAY( mWTemplList );
	WX_CLEAR_ARRAY( mPropSets );
}

bool ResTemplateManager::LoadGroupData( ResPropSetInfo& set, Record& fromRec, ResGroupData& data )
{
	SetGroupValuesRecursive( data, set, fromRec );

	return TRUE;
}

bool ResTemplateManager::StoreGroupData( ResPropSetInfo& set, Record& toRec, ResGroupData& data )
{
	toRec.RemoveAllFields();

	GetGroupValuesRecursive( data, set, toRec );

	return TRUE;
}

bool ResTemplateManager::IsWidgetRecord( Record* pRec )
{
	return FindWidgetTemplateForTag( pRec->GetName() ) != NULL;
}

bool ResTemplateManager::IsKnownTag( const wxString& tag )
{
	return mTagHash.Get( tag ) != NULL;
}

WidgetResTemplate* ResTemplateManager::FindWidgetTemplateForTag( const wxString& tag )
{
	WidgetResTemplate* pTempl =  (WidgetResTemplate*)mTagHash.Get( tag );

	if ( !pTempl )

		return &mUnknownWidgetTempl;
	else
		return pTempl;
}

bool ResTemplateManager::LoadResourceTemplates( const wxString& fname )
{
	WX_CLEAR_ARRAY( mWTemplList );
	WX_CLEAR_ARRAY( mPropSets );

	TextFileRecordSet rset( fname );

	rset.MoveFirst();

	while( !rset.IsEof() )
	{
		Record& rec = rset.GetRecord();

		if ( rec.GetName() == "prop_set" )
		{
			ResPropSetInfo* pSet = new ResPropSetInfo();

			mPropSets.Add( pSet );

			LoadPropSetInfo( rec, *pSet );
		}
		else
		if ( rec.GetName() == "widget" || rec.GetName() == "unknown_widget" )
		{
			bool isUnknown = rec.GetName() == "unknown_widget";

			WidgetResTemplate* pTempl = 
				( isUnknown ) ? &mUnknownWidgetTempl 
							  : new WidgetResTemplate();

			pTempl->mIsUnknown = isUnknown;

			pTempl->mTag = rec[ "tag" ];

			pTempl->mCanHaveChildren = FALSE;

			if ( rec.FindField( "can_have_children" ) && 
				 rec[ "can_have_children" ] == "true" )

				 pTempl->mCanHaveChildren = TRUE;

			pTempl->mDisplayNameExpr = rec[ "display_name_expr" ];

			if ( rec.FindField( "previewer_class" ) )
				
				pTempl->mPreviewerClass = rec[ "previewer_class" ];
			else
				pTempl->mPreviewerClass = "WidgetGlyph";

			if ( rec.FindField( "node_class" ) )
				
				pTempl->mNodeClass = rec[ "node_class" ];
			else
				pTempl->mNodeClass = "WidgetNode";

			if ( rec.FindField( "widget_class" ) )
				
				pTempl->mWidgetClass = rec[ "widget_class" ];
			else
				pTempl->mWidgetClass = "wxWindow";

			if ( rec.FindField( "toolbar_icon" ) )
			{
				if ( pTempl->mTag == "frame" )
				{
					int o;
					++o;
				}

				pTempl->mToolBarBitmap = 
					wxUILookRenderer::RecordToBitmap( &rec.GetRecordByName( "toolbar_icon" ) );

				int w= pTempl->mToolBarBitmap.GetWidth(), h = pTempl->mToolBarBitmap.GetHeight();
			}

			pTempl->mpWidgetAttrGroup = new wxDMAttrGroupInfo();
			pTempl->mpExtraAttrGroup = NULL;

			wxASSERT_MSG( rec.FindField( "props" ), "props field not found in widget description!" ); // DBG::

			LoadPropSetInfo( rec, pTempl->mPropSet );

			int curIdx = 0;

			if ( rec.FindField( "extra_props" ) )
			{
				LoadPropSetInfo( rec, pTempl->mExtraPropSet, "extra_props" );

				pTempl->mExtraPropsTagName = rec[ "extra_props_tag_name" ];

				pTempl->mpExtraAttrGroup = new wxDMAttrGroupInfo();

				CreateGroupRecursive( *pTempl->mpExtraAttrGroup , pTempl->mExtraPropSet, curIdx );

				pTempl->mpExtraAttrGroup ->SetName( rec[ "extra_props_name" ] );

			}

			curIdx = 0;

			CreateGroupRecursive( *pTempl->mpWidgetAttrGroup, pTempl->mPropSet, curIdx );

			pTempl->mpWidgetAttrGroup->SetName( "Properties" );

			if ( !isUnknown )
			{
				mWTemplList.Add( pTempl );
				mTagHash.Put( pTempl->mTag, (wxObject*)pTempl );
			}
		}
		else
			wxASSERT_MSG(0, "Unknown tag : " + rec.GetName() ); // DBG:: bogous record name

		rset.MoveNext();
	}

	return TRUE;
}

WidgetResTemplateListT& ResTemplateManager::GetTemplates()
{
	return mWTemplList;
}

WidgetResTemplate& ResTemplateManager::GetUnknownWidgetTemplate()
{
	return mUnknownWidgetTempl;
}

/*** protected methods ***/

void ResTemplateManager::CreateGroupRecursive( wxDMAttrGroupInfo& group, 
											   ResPropSetInfo& set, 
											   int& curIndex )
{
    size_t i;
	for( i = 0; i != set.mProps.Count(); ++i )
	{
		ResPropInfo& pr = *set.mProps[i];

		wxDMAttrInfo* pAttr = NULL; int type = pr.GetType();

		if ( type == RESPROP_NUMBER )
							
			pAttr = new ResIntAttrInfo();
		else
		if ( type == RESPROP_BOOLEAN )
							
			pAttr = new ResBoolAttrInfo();
		else
		if ( type == RESPROP_STRING )
		
			pAttr = new ResStringAttrInfo();
		else
		if ( type == RESPROP_COLOR )

			pAttr = new ResColorAttrInfo();
		else
		if ( type == RESPROP_RESTRICTED_STRING ||
			 type == RESPROP_RESTRICTED_STRING_AS_BOOLEAN )
		{
			pAttr = new ResRestrictedStringAttrInfo();

			pAttr->SetRestrictedValues( pr.mRestrDispValues );

			if ( type == RESPROP_RESTRICTED_STRING_AS_BOOLEAN )

				((ResRestrictedStringAttrInfo*)pAttr)->SetShownAsBoolean( TRUE );
		}
		else
		if ( type == RESPROP_STRING_LIST )
		
			pAttr = new ResStringListAttrInfo();
		else
		if ( type == RESPROP_PROPSET )
		{
			/* recursive invocation */

			ResPropSetInfo* pNested = (ResPropSetInfo*)&pr;
			CreateGroupRecursive( group, *pNested, curIndex );
		}
		else
		if ( type == RESPORP_CONCAT_PROPS )
		{
			// nothing to do with these during creation
		}
		else
			wxASSERT(0); // DBG:: bogous type of property

		if ( pAttr )
		{
			pAttr->SetUserData( (void*)curIndex );
			pAttr->SetName( pr.mName );
			pAttr->SetDisplayName( pr.mDispName );
			pAttr->SetDefaultValue( pr.mDefaultValue );
			group.Add( pAttr );

			++curIndex;
		}
	}
}

void ResTemplateManager::SetGroupValuesRecursive( ResGroupData& groupData,
												  ResPropSetInfo& set,
												  Record& rec )
{
	if ( set.mGluedWith != "" )
	{
		wxString value = ""; StrListT lst;

		if ( rec.FindField( set.mName ) )

			value = rec[ set.mName ];

		ExtractConcatinatedValues( value, lst, set.mGluedWith[0] );

        size_t i;
		for( i = 0; i != set.mProps.Count(); ++i )
		{
			ResPropInfo& pr = *set.mProps[i];

			wxDMAttrInfo* pAttr = groupData.GetGroupInfo()->GetAttrByName( pr.mName ); 

			if ( i < (unsigned int)lst.Count() )
				pAttr->SetStringValue( lst[i], &groupData );
			else
				pAttr->SetStringValue( pr.mDefaultValue, &groupData );
		}

		return;
	}

    size_t i;
	for( i = 0; i != set.mProps.Count(); ++i )
	{
		ResPropInfo& pr = *set.mProps[i];

		wxDMAttrInfo* pAttr = groupData.GetGroupInfo()->GetAttrByName( pr.mName ); 
		int type = pr.GetType();
		void* pValue = NULL;

		if ( !pr.mVisibileOnly &&
			 ( type == RESPROP_NUMBER  || 
			   type == RESPROP_STRING  ||
			   type == RESPROP_BOOLEAN || 
			   type == RESPROP_COLOR   ||
			   type == RESPROP_RESTRICTED_STRING ||
			   type == RESPROP_RESTRICTED_STRING_AS_BOOLEAN )
		   )
		{					
			wxString value = pr.mDefaultValue;

			if ( rec.FindField( pr.mName ) )

				value = rec[ pr.mName ];

			if ( type == RESPROP_RESTRICTED_STRING ||
				 type == RESPROP_RESTRICTED_STRING_AS_BOOLEAN )
			
				value = pr.TranslateToDispValue( value );

			pAttr->SetStringValue( value, &groupData );
		}
		else
		if ( type == RESPROP_STRING_LIST )
		{
			wxStringList lst;

            size_t i;
			for( i = 0; i != rec.GetFieldCount(); ++i )
			{
				FieldBase* pField = &rec.GetFieldByIndex(i);

				if ( pField->GetType() == FT_RECORD &&
					 pField->GetName() == "item" )
				
					lst.Add( ((Record*)pField)->GetStringValue() );
			}

			pAttr->SetMultipleValues( lst, &groupData );
		}
		else
		if ( type == RESPROP_PROPSET )
		{
			/* recursive invocation */

			ResPropSetInfo* pNested = (ResPropSetInfo*)&pr;

			SetGroupValuesRecursive( groupData, *pNested, rec );
		}
		else
		if ( type == RESPORP_CONCAT_PROPS )
		{
			ResPropSetInfo* pNested = (ResPropSetInfo*)&pr;

			// redistribut values concatinated

			wxString concValue = rec[ pr.mName ];
			StrListT lst; ExtractConcatinatedValues( concValue, lst );

            size_t n;
			for( n = 0; n != (unsigned int)lst.Count(); ++n )

				lst[n].Trim( TRUE ).Trim( FALSE );

            size_t i;
			for( i = 0; i != (unsigned int)pNested->mProps.Count(); ++i )
			{	 
				ResPropInfo& prop = *pNested->mProps[i];

                size_t k;
				for( k = 0; k != (unsigned int)lst.Count(); ++k )
				
					if ( prop.HasRestrictedValue( lst[k] ) )
					{
						size_t idx = groupData.GetGroupInfo()->GetAttrIndex( prop.mName );
						wxASSERT( idx != (size_t)(-1) ); // DBG::

						wxString displayValue = prop.TranslateToDispValue( lst[k] );

						groupData.GetGroupInfo()->GetAttr( idx )->
							SetStringValue( displayValue, &groupData );

						break;
					}
			}
		}
		else
		if ( !pr.mVisibileOnly )

			wxASSERT(0); // DBG:: bogous type of property
	}
}

void ResTemplateManager::GetGroupValuesRecursive( ResGroupData& groupData,
												  ResPropSetInfo& set,
												  Record& rec )
{
	if ( set.mGluedWith != "" )
	{
		wxString value;

        size_t i;
		for( i = 0; i != set.mProps.Count(); ++i )
		{
			ResPropInfo& pr = *set.mProps[i];

			wxDMAttrInfo* pAttr = groupData.GetGroupInfo()->GetAttrByName( pr.mName ); 

			if ( i != 0 ) value += set.mGluedWith;

			value += pAttr->GetStringValue( &groupData );
		}

		rec.AddRecord( set.mName, value );

		return;
	}

    size_t i;
	for( i = 0; i != set.mProps.Count(); ++i )
	{
		ResPropInfo& pr = *set.mProps[i];

		wxDMAttrInfo* pAttr = groupData.GetGroupInfo()->GetAttrByName( pr.mName ); 
		int type = pr.GetType();
		void* pValue = NULL;

		if ( type == RESPROP_NUMBER  || 
			 type == RESPROP_BOOLEAN || 
			 type == RESPROP_STRING  ||
			 type == RESPROP_COLOR   ||
			 type == RESPROP_RESTRICTED_STRING ||
			 type == RESPROP_RESTRICTED_STRING_AS_BOOLEAN )
		{
			wxString value = pAttr->GetStringValue( &groupData );

			if ( type == RESPROP_RESTRICTED_STRING ||
				 type == RESPROP_RESTRICTED_STRING_AS_BOOLEAN )

				value = pr.TranslateFromDispValue( value );

			if ( pr.mVisibileOnly == FALSE )
			{
				// FIXME ME:: FOR NOW:: according to wxXml-res format, only "name" goes as 
				//						attribute values, all the rest goes as child-element nodes

				if ( type == RESPROP_STRING &&  pr.mName == "name" )

					rec.AddStringField( pr.mName, value );
				else
				if ( value != pr.mDefaultValue )

					rec.AddRecord( pr.mName, value );
			}

		}
		if ( type == RESPROP_STRING_LIST && pr.mVisibileOnly == FALSE )
		{
			// HACK:: hardcoded

			wxStringList lst;
			pAttr->GetMultipleValues( lst, &groupData );

            size_t i;
			for( i = 0; i != lst.GetCount(); ++i )
			
				rec.AddRecord( "item", lst[i] );
		}
		else
		if ( type == RESPROP_PROPSET )
		{
			ResPropSetInfo* pNested = (ResPropSetInfo*)&pr;
			GetGroupValuesRecursive( groupData, *pNested, rec );
		}
		else
		if ( type == RESPORP_CONCAT_PROPS )
		{
			ResPropSetInfo* pNested = (ResPropSetInfo*)&pr;

			// collect values distributed among multiple properties

			wxString concValue;
			size_t nValues = 0;

			// TBD:: default value

            size_t i;
			for( i = 0; i != pNested->mProps.Count(); ++i )
			{	 
				ResPropInfo& prop = *pNested->mProps[i];

				wxDMAttrInfo* pAttr = groupData.GetGroupInfo()->GetAttrByName( prop.mName );

				wxString value = pAttr->GetStringValue( &groupData );
				value = prop.TranslateFromDispValue( value );

				if ( pAttr && value != "" )
				{
					if ( nValues > 0 ) concValue += "|";

					concValue += value;

					++nValues;
				}
			}

			if ( concValue != "" )

				rec.AddRecord( pr.mName, concValue );
		}
	}
}

void ResTemplateManager::LoadPropSetInfo( Record& rec, ResPropSetInfo& set, 
										  const wxString& propsTag )
{
	set.mName = rec[ "name" ];
	set.mType = RESPROP_PROPSET;

	if ( rec.FindField( "glued_with" ) )

		set.mGluedWith = rec[ "glued_with" ];

	if ( rec.FindField( propsTag ) )
	{
		Record& props = rec.GetRecordByName( propsTag );

        size_t i;
		for( i = 0; i != props.GetFieldCount(); ++i )
		{
			FieldBase* pField = &props.GetFieldByIndex(i);

			if ( pField->GetType() == FT_RECORD )
			{
				ResPropInfo* pProp = new ResPropInfo();

				pProp->mName = pField->GetName();
				pProp->mDispName = pProp->mName;

				// referes to atomic property

				Record& r = *(Record*)pField;

				wxString type = r[0]; size_t idx = 0;

				// first, property type declarator expected

				if ( type == "NUMBER" ) pProp->mType = RESPROP_NUMBER;
					else
				if ( type == "BOOLEAN" ) pProp->mType = RESPROP_BOOLEAN;
					else
				if ( type == "STRING" ) pProp->mType = RESPROP_STRING;
					else
				if ( type == "COLOR") pProp->mType = RESPROP_COLOR;
					else
				if ( type == "STRING_LIST" ) pProp->mType = RESPROP_STRING_LIST;
					else
				if ( type == "FLAG" || type == "BOOLEAN_FLAG"  ) 
				{
					// list of retricted display values expected

					pProp->mType = RESPROP_RESTRICTED_STRING;

					if ( type == "BOOLEAN_FLAG" )

						pProp->mType = RESPROP_RESTRICTED_STRING_AS_BOOLEAN;

					++idx;
					StrListT lst;
					r.GetRecordByIndex(idx).CopyStringValues( lst );

					::StrListToStringList( pProp->mRestrDispValues, lst );

					if ( r.FieldAt(2).GetType() == FT_RECORD )
					{
						// optional list of actual values if present

						++idx; lst.Clear();
						r.GetRecordByIndex(idx).CopyStringValues( lst );

						::StrListToStringList( pProp->mRestrValues, lst );
					}
					else
						// otherwise actual values are equal to display values

						pProp->mRestrValues = pProp->mRestrDispValues;
				}

				++idx;

				// default value of the property, if any

				if ( idx < r.GetFieldCount() )
				{
					pProp->mDefaultValue = r[idx]; ++idx;
				}

				// display-name of the property, if any

				if ( idx < r.GetFieldCount() )
				{
					pProp->mDispName = r[ idx ]; ++idx;
				}

				// _VISIBLE_ONLY_ flag, if any

				if ( idx < r.GetFieldCount() && r[ idx ] == "_VISIBLE_ONLY_" )
				{
					pProp->mVisibileOnly = TRUE;
				}

				set.mProps.Add( pProp );
			}
			else
			{
				StringValue& strField = *(StringValue*)pField;

				if ( strField.GetName() == "" )
				{
					// referes to the nested property-set

                    size_t k;
					for( k = 0; k != mPropSets.Count(); ++k )

						if ( mPropSets[k]->mName == strField.GetValue() )
						{
							set.mProps.Add( mPropSets[k] );
							break;
						}
				}
				else
				{
					// referes to concatinated properties

					ResPropSetInfo* pConcSet = new ResPropSetInfo();
					pConcSet->mName = strField.GetName();
					pConcSet->mType = RESPORP_CONCAT_PROPS;

					wxString concProps = strField.GetValue(); StrListT lst;

					ExtractConcatinatedValues( concProps, lst );

                    size_t n;
					for( n = 0; n != (unsigned int)lst.Count(); ++n )
					{
						wxString name = lst[n];

						ResPropInfo* pFound = SearchPropertyTree( set, name );

						if ( pFound )

							pConcSet->mProps.Add( pFound );
					}

					set.mProps.Add( pConcSet );
				}
			}
		}
	}
}

ResPropInfo* ResTemplateManager::SearchPropertyTree( ResPropSetInfo& set, 
													 const wxString& propName )
{
    size_t i;
	for( i = 0; i != set.mProps.Count(); ++i )
	{
		ResPropInfo* pInfo = set.mProps[i];

		if ( pInfo->mType == RESPROP_PROPSET )
		{
			ResPropInfo* pFound = SearchPropertyTree( *(ResPropSetInfo*)pInfo, propName );

			if ( pFound ) return pFound;
		}
		else
		if ( pInfo->mType == RESPORP_CONCAT_PROPS )
		{
			// nothing
		}
		else
		{
			// atomic property

			if ( pInfo->mName == propName )

				return pInfo;
		}
	}

	return NULL;
}