/////////////////////////////////////////////////////////////////////////////
// Name:        No names yet.
// Purpose:     Contrib. demo
// Author:      Aleksandras Gluchovas
// Modified by:
// Created:     2000/02/08
// RCS-ID:      
// Copyright:   (c) Aleksandras Gluchovas (alex@soften.ktu.lt)
// Licence:   	GPL (version 2.0 or later)
/////////////////////////////////////////////////////////////////////////////

#ifdef __GNUG__
    #pragma implementation "minimal.cpp"
    #pragma interface "minimal.cpp"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include "widgeteditor.h"

#include "wx/choicdlg.h"

// X-Framework stuff
#include "XObject.h"
#include "XCellBackground.h"
#include "XSelectionEditor.h"
#include "XRectEditor.h"

#include "propedit.h"
#include "dmbrowser.h"
#include "gedthandler.h"
#include "myutils.h"

// sorry, 4 .cpp's and .h's of AWT-layouts contrib. needed to compile this .cpp!

#include "wx/awtlay/gridbaglayout.h"
#include "wx/awtlay/flowlayout.h"
#include "wx/awtlay/boxlayout.h"
#include "wx/awtlay/gridlayout.h"

/***** Implementation for class WidgetDocument *****/

IMPLEMENT_DYNAMIC_CLASS( WidgetDocument, wxPMDocument )

WidgetDocument::WidgetDocument()

	: mpRootNode( NULL ),
	  mpRecord( NULL ),
	  mpDMCmdProcessor( NULL ),
	  mpRenderer( NULL ),
	  mInsertionOfs( 15,15 ),
	  mIdsHash( wxKEY_STRING )
{
#if defined(__WINDOWS__)
	mUIType = "WINDOWS";
#elif defined(__WXGTK__)
	mUIType = "GTK";
#elif defined(__WXPM__)
	mUIType = "PM";
#elif defined(__WXMOTIF__)
	mUIType = "MOTIF";
#elif defined(__WXQT__)
	mUIType == "QT";
#elif defined(__WXMAC__)
	mUIType == "MAC";
#else
	mUIType == "UNKNOWN";
#endif

	mpRenderer = new wxUILookRenderer();

	if ( !mpRenderer->ParseScriptFromFile( "config_stdwlooks.wlk" ) )
	
		wxMessageBox( "Error while loading widget-looks from \"config_stdwlooks.wlk\"!" );
}

WidgetDocument::~WidgetDocument()
{
	if ( mpRootNode )

		delete mpRootNode;
}

bool WidgetDocument::CanHaveFile()
{
	return FALSE;
}

bool WidgetDocument::SavePMDocument()
{
	ResDocument& doc = *((ResDocument*)GetParentDocument());

	StoreNodesRecursively( mpRootNode, *mpRecord, doc.GetTemplateManager() );

	if ( IsModified() )

		// parent's data becomes dirty after child's modifications were saved
		GetParentDocument()->Modify( TRUE );

	return TRUE;
	// TBD::
}

void WidgetDocument::SetRootNode( wxDMNode* pRoot )
{
	mpRootNode = (WidgetNode*)pRoot;
}

wxDMNode* WidgetDocument::GetRootNode()
{
	return mpRootNode;
}

WidgetResTemplate* WidgetDocument::GetTemplateForNode( wxDMNode& node )
{
	WidgetNode& wnode = *(WidgetNode*)&node;

	return &wnode.GetTemplate();
}

ResTemplateManager& WidgetDocument::GetTemplateManager()
{
	ResDocument& doc = *((ResDocument*)GetParentDocument());

	return doc.GetTemplateManager();
}

bool WidgetDocument::LoadPMDocument()
{
	ResDocument& doc = *((ResDocument*)GetParentDocument());

	mpRecord = doc.GetCurrentRecord();

	if ( mpRootNode ) delete mpRootNode;

	bool ok = LoadNodesRecursively( *mpRecord, 
								 NULL, NULL, doc.GetTemplateManager() );

	UpdateAllViews();

	return ok;
}

bool WidgetDocument::IsModified() const
{
	return mpRootNode->FindCmdProcessor()->CanUndo();
}

void WidgetDocument::Modify(bool modify)
{
	if ( !modify )
	
		mpRootNode->FindCmdProcessor()->ClearCommandBuffer();
}

WidgetNode* WidgetDocument::InsertWidgetByCode( int code, 
											    WidgetNode* pTarget, 
												const XPoint& pos,
												int zOrder )
{
	WidgetResTemplate* pTempl = GetTemplateManager().GetTemplates()[ code ];

	wxClassInfo* pClass = wxClassInfo::FindClass( 
								(char*)(const char*)pTempl->mNodeClass );

	WidgetNode* pNode = NULL;

	if ( pClass ) pNode = (WidgetNode*)(pClass->CreateObject());

	if ( !( pClass && pNode ) )
	{
		wxMessageBox( "Cannot instantiation widget-node object of class" +
					  pTempl->mNodeClass );

		return FALSE;
	}

	pNode->mpTempl = pTempl;
	pNode->mpView = (WidgetView*)GetFirstView();

	pNode->mAttribGroups[ WidgetNode::WIDGET_PROPERTIES_GROUP ].
		SetGroupInfo( pTempl->mpWidgetAttrGroup );

	pNode->mBounds.x = pos.x;
	pNode->mBounds.y = pos.y;

	// now obtain the natural (preferred) dimension of the inserted widget

	wxUILookInfo* pInfo = FindLookForNode( *pNode );

	if ( pInfo )
	{
		// HACK::
		wxClientDC dc( wxPMBootSvc().GetMainWindow() );

		wxString text;
		int textWidth = 0, textHeight = 0,
			textX = 0, textY = 0,
			itemWidth = 0, itemHeight = 0;

		mSolver.Init( pNode->GetAttrGroupInfo( WidgetNode::WIDGET_PROPERTIES_GROUP ),
					  pNode->GetAttrGroupData( WidgetNode::WIDGET_PROPERTIES_GROUP ),
					  mUIType );

		mpRenderer->CalcItemParams( *pInfo, wxSize(10,10), dc, text, 
									textWidth, textHeight, 
									textX, textY, itemWidth, itemHeight, &mSolver );

		pNode->mBounds.width  = itemWidth;
		pNode->mBounds.height = itemHeight;
	}
	else
	{
		//FIXME:: ??
		pNode->mBounds.width  = 30;
		pNode->mBounds.height = 30;
	}

	pNode->StoreBoundsInfo();

	wxString newId = GenerateUniqueId( *pNode );

	pNode->SetNodeAttrValue( "name", newId );

	pTarget->AddChild( pNode, zOrder );

	wxDC& dc = pNode->mpView->GetScene().GetCanvas();

	pNode->NotifyAdded( pNode );

	/*
	if ( pTarget->IsKindOf( CLASSINFO( SizerNodeBase ) ) )
	{
		((SizerNodeBase*)pTarget)->AddExtraPropertiesToChild( pNode );

		((WidgetNode*)&pNode->mpView->GetRootNode())->DoLayoutChildren( dc );
	} */

	return pNode;
}

bool WidgetDocument::FindInsertionPoint( XPoint& pos, WidgetNode*& pTarget )
{
	wxASSERT( mpRootNode ); // DBG::

	DMNodeListT lst;
	mpRootNode->GetSelection( lst );

	pos.x = 10000; pos.y = 10000; pTarget = NULL;

	for( size_t i = 0; i != lst.Count(); ++i )
	{
		WidgetNode* pNode = (WidgetNode*)lst[i];

		XRect& bounds = ((WidgetNode*)lst[i])->GetBounds();

		pos.x = wxMin( pos.x, bounds.x );
		pos.y = wxMin( pos.y, bounds.y );
	}

	if ( lst.Count() > 1 )

		pTarget = (WidgetNode*)lst[0]->GetParent();
	else
	{
		if ( lst.Count() == 0 )
		{
			pos.x = mpRootNode->GetBounds().x;
			pos.y = mpRootNode->GetBounds().y;

			pTarget = mpRootNode;
		}
		else
		{
			WidgetNode* pNode = (WidgetNode*)lst[0];

			if ( pNode->mpTempl->mCanHaveChildren )
			{
				pos.x = pNode->GetBounds().x;
				pos.y = pNode->GetBounds().y;

				pTarget = pNode;
			}
			else
				pTarget = (WidgetNode*)pNode->GetParent();
		}
	}

	pos.x += mInsertionOfs.x;
	pos.y += mInsertionOfs.y;

	return TRUE;
}

wxUILookInfo* WidgetDocument::FindLookForNode( WidgetNode& node )
{
	mSolver.Init( node.GetAttrGroupInfo( WidgetNode::WIDGET_PROPERTIES_GROUP ),
				  node.GetAttrGroupData( WidgetNode::WIDGET_PROPERTIES_GROUP ),
				  mUIType );


	UILookInfoListT& lst = mpRenderer->GetInfoItems();
	EqSolverVar result;

	// first, try quick way

	const wxString& widgetClass = node.GetWidgetClass();

	size_t i = 0;
	for( ; i != lst.Count(); ++i )
	{
		if ( widgetClass == lst[i]->mClassHint )
		{
			wxUILookInfo& info = *lst[i];

			info.mpStyleFilterExprNode->Evaluate( result, mSolver );

			if ( result.GetBool() == TRUE )

				return &info;
		}
	}

	// otherwise do full-scan of look-infos without class-hints,
	// executing everyone's filter-expression

	for( i = 0; i != lst.Count(); ++i )
	{
		wxUILookInfo& info = *lst[i];

		if ( info.mClassHint.Length() == 0 )
		{
			info.mpStyleFilterExprNode->Evaluate( result, mSolver );

			if ( result.GetBool() == TRUE )

				return &info;
		}
	}

	return NULL;
}

wxUILookRenderer& WidgetDocument::GetRenderer()
{
	return *mpRenderer;
}

const wxString& WidgetDocument::GetUIType()
{
	return mUIType;
}

void WidgetDocument::SetUIType( const wxString& uiType )
{
	mUIType = uiType;
}

void WidgetDocument::MarkUsedId( const wxString& id )
{
	mIdsHash.Put( id, (wxObject*)this );
}

void WidgetDocument::MarkUnusedId( const wxString& id )
{
	mIdsHash.Delete( id );
}

wxString WidgetDocument::GenerateUniqueId( WidgetNode& forNode )
{
	for( int i = 1;; ++i )
	{
		wxString id = forNode.mpTempl->mTag + IntToStr( i );

		if ( mIdsHash.Get( id ) == NULL )

			return id;
	}
}

WidgetEqSolver& WidgetDocument::GetEqSolver()
{
	return mSolver;
}

/*** protected methods ***/

bool WidgetDocument::LoadNodesRecursively( Record& rec, Record* pExtraPropsRec,
										   WidgetNode* pParent, 
										   ResTemplateManager& mgr )
{
	ResTemplateManager& templMgr = GetTemplateManager();

	WidgetResTemplate* pTempl = mgr.FindWidgetTemplateForTag( rec.GetName() );

	if ( pTempl->mTag == "unknown" )
	{
		int o = 0;
		++o;
	}

	// widget-preview class is provided explicitely in the script,
	// here we try to find and instantiate it

	wxClassInfo* pClass = wxClassInfo::FindClass( 
								(char*)(const char*)pTempl->mNodeClass );

	WidgetNode* pNode = NULL;

	if ( pClass ) pNode = (WidgetNode*)(pClass->CreateObject());

	if ( !( pClass && pNode ) )
	{
		wxMessageBox( "Cannot instantiation widget-node object of class" +
					  pTempl->mNodeClass + " for tag " + rec.GetName() );

		return FALSE;
	}

	if ( pParent )

		pParent->AddChild( pNode );
	else
	{
		SetRootNode( pNode );

		// root node alaws contains command-processor

		mpDMCmdProcessor = new wxDMCommandProcessor();

		mpDMCmdProcessor->SetClipboard( 
			&((ResDocument*)GetParentDocument())->GetClipboard() );

		pNode->SetCommandProcessor( mpDMCmdProcessor );

		mpDMCmdProcessor->SetRoot( pNode );
	}

	pNode->mpTempl = pTempl;
	pNode->mpView = (WidgetView*)GetFirstView();

	pNode->mAttribGroups[ WidgetNode::WIDGET_PROPERTIES_GROUP ].
		SetGroupInfo( pTempl->mpWidgetAttrGroup );

	mgr.LoadGroupData( pTempl->mPropSet, rec,
					   pNode->mAttribGroups[ WidgetNode::WIDGET_PROPERTIES_GROUP ] );

	// layout-props record always comes from node's parent
	if ( pExtraPropsRec && pParent->mpTempl->mpExtraAttrGroup )
	{
		pNode->mAttribGroups[ WidgetNode::EXTRA_PROPERTIES_GROUP ].
			SetGroupInfo( pParent->mpTempl->mpExtraAttrGroup );

		mgr.LoadGroupData( pParent->mpTempl->mExtraPropSet, *pExtraPropsRec,
						   pNode->mAttribGroups[ WidgetNode::EXTRA_PROPERTIES_GROUP ] );
	}

	// retriev x,y,width,height, i.e. the "dynamic-properties" and 
	// store them to mBounds structure for efficient access while layouting

	pNode->ExtractBoundsInfo();

	wxString extraPropsTag;

	// scan child-fields for composite records, which
	// should normally describe item's children

	for( size_t i = 0; i != rec.GetFieldCount(); ++i )
	{
		wxString& name = rec.GetFieldByIndex(i).GetName();

		// check if item is "wrapped up" into node which additionally
		// contains extra-properties that were assigned by item's parent

		if ( name == pNode->mpTempl->mExtraPropsTagName )
		{
			Record& extraPropsRec = rec.GetRecordByIndex(i);

			for( size_t k = 0; k != extraPropsRec.GetFieldCount(); ++k )
			{
				Record& childRec = extraPropsRec.GetRecordByIndex(k);

				if ( templMgr.IsKnownTag( extraPropsRec.GetFieldByIndex(k).GetName() ) )
				{
					if ( !LoadNodesRecursively( extraPropsRec.GetRecordByIndex(k), 
												&extraPropsRec, pNode, mgr ) )

						return FALSE;
				}
			}
		}
		else
		if ( templMgr.IsKnownTag( name ) )
		{
			if ( !LoadNodesRecursively( rec.GetRecordByIndex(i), NULL,
										pNode, mgr ) 
			   )
				return FALSE;
		}
	}

	return TRUE;
}

bool WidgetDocument::StoreNodesRecursively( WidgetNode* pNode, Record& rec,
											ResTemplateManager& mgr )
{
	Record* pRec = NULL;

	// if there any extra-properties (assigned by parent, eg. 
	// sizing-parameters to each child of BoxSizer) attached to 
	// the item, then it's parent will be record containg these properties
	// as fields, and the actual item-record as it's child

	if ( pNode->HasExtraProperties() )
	{
		// create parent container-node which will enclose
		// the child togeather with extra-properties

		WidgetResTemplate& parentTempl = *((WidgetNode*)pNode->GetParent())->mpTempl;

		rec.SetName( parentTempl.mExtraPropsTagName );

		mgr.StoreGroupData( parentTempl.mExtraPropSet, rec, 
							pNode->mAttribGroups[ WidgetNode::EXTRA_PROPERTIES_GROUP ] );

		pRec = new Record();

		rec.AddField( pRec ); 
	}
	else
		pRec = &rec; // otherwise store child's properties to the given node

	WidgetResTemplate& templ = *pNode->mpTempl;

	pRec->RemoveAllFields();
	pRec->SetName( templ.mTag );

	// store values of mBounds to  x,y,width,height "dynamic" properties 
	pNode->StoreBoundsInfo();

	mgr.StoreGroupData( templ.mPropSet, *pRec, 
						pNode->mAttribGroups[ WidgetNode::WIDGET_PROPERTIES_GROUP ] );

	for( size_t i = 0; i != pNode->GetChildCount(); ++i )
	{
		Record* pChildRec = new Record();
		pRec->AddField( pChildRec );

		StoreNodesRecursively( (WidgetNode*)pNode->GetChild(i), *pChildRec, mgr );
	}

	return TRUE;
}

/***** Implementation for class WidgetView *****/

static wxCommandInfo _gCanvasCommands[] = {

	wxCommandInfo( WidgetView::ID_SET_UI_TYPE, "Set UI-&Type...", "", "Ctrl+T" ), 
	wxCommandInfo( WidgetView::ID_SET_UNITS, "Set &Units...", "", "" ), 
	wxCommandInfo( WidgetView::ID_SET_SIZER, "Set &Sizer...", "", "" ), 
	wxCommandInfo( -1, "SEPARATOR" ), 

	wxCommandInfo( WidgetView::ID_INSERT_WIDGET, "&Insert Widget...", "", "" ), 
	wxCommandInfo( WidgetView::ID_GRID_SETTINGS, "&Grid Settings...", "", "Ctrl+G" ), 
	wxCommandInfo( WidgetView::ID_RELOAD_WIDGET_LOOKS, "&Reload widget looks", "", "" ), 
	wxCommandInfo( WidgetView::ID_PREVIEW, "&Preview...", "", "" ), 
	wxCommandInfo( -1, "SEPARATOR" ), 
	wxCommandInfo( WidgetView::ID_SHOW_SIZERS, "&Highlight sizers", "", "", TRUE, TRUE ), 

	wxCommandInfo( 0, "" ) // terminator
};

wxDMClipboard* WidgetView::mpClipboard = NULL;

IMPLEMENT_DYNAMIC_CLASS( WidgetView, wxPMView )

BEGIN_EVENT_TABLE( WidgetView, wxPMView )

	EVT_COMMAND_RANGE( XGenericEditorHandler::FILE_OPEN,
					   XGenericEditorHandler::LAYOUT_FLIP,
					   wxEVT_COMMAND_MENU_SELECTED,
					   WidgetView::OnEditorCommandsRange )

	EVT_COMMAND_RANGE( XGenericEditorHandler::FILE_OPEN,
					   XGenericEditorHandler::LAYOUT_FLIP,
					   wxEVT_UPDATE_UI,
					   WidgetView::OnEditorCommandsRange )

	EVT_COMMAND_RANGE( ID_INSERT_WIDGET_IDS_START,
					   ID_INSERT_WIDGET_IDS_END,
					   wxEVT_COMMAND_MENU_SELECTED,
					   WidgetView::OnInsertWidgetCommandsRange )

	EVT_MENU( ID_SET_UI_TYPE, WidgetView::OnSetUIType )
	EVT_MENU( ID_SET_UNITS, WidgetView::OnSetUnits )
	EVT_MENU( ID_SET_SIZER, WidgetView::OnSetSizer )
	EVT_MENU( ID_GRID_SETTINGS, WidgetView::OnGridSettings )
	EVT_MENU( ID_INSERT_WIDGET, WidgetView::OnInsertWidget )
	EVT_MENU( ID_PREVIEW, WidgetView::OnPreview )
	EVT_MENU( ID_RELOAD_WIDGET_LOOKS, WidgetView::OnReloadWigetLooks )
	EVT_MENU( ID_SHOW_SIZERS, WidgetView::OnShowSizers )

END_EVENT_TABLE()

WidgetView::WidgetView()

	: mpScene( NULL ),
	  mpEditorHandler( NULL ),
	  mpXCmdMgr( NULL ),
	  mpRootNode( NULL ),
	  mUseDialogUnits( FALSE ),
	  mSizersShown( TRUE ),
	  mGridType( GRID_NONE ),
	  mGridSpacing( 10,10 ),
	  mSnapSpacing( 1,1 ),
	  mpWidgetMenus( NULL ),
	  mpMenuChains( NULL ),
	  mpPropPane( NULL )
{
	mpEditorHandler = new XGenericEditorHandler();
}

WidgetView::~WidgetView()
{
	delete mpEditorHandler;

	if ( mpScene ) 
	{
		delete mpScene;
		delete mpXCmdMgr;
		GetViewFrame()->PopEventHandler();
	}
}

XCollection& WidgetView::GetScene()
{
	return *mpScene;
}

void WidgetView::SetDialogsUnits( bool on )
{
	mUseDialogUnits = on;
}

bool WidgetView::UseDialogUnits()
{
	return mUseDialogUnits;
}

WidgetGlyph& WidgetView::GetGlyphForNode( wxDMNode& node )
{
	WidgetGlyph* pGlyph = (WidgetGlyph*)mGlyphHash.Get( (long)&node );

	wxASSERT( pGlyph ); // DBG::

	return *pGlyph;
}

WidgetGlyph* WidgetView::FindGlyphForNode( wxDMNode& node )
{
	return (WidgetGlyph*)mGlyphHash.Get( (long)&node );
}

WidgetNode& WidgetView::GetRootNode()
{
	return *mpRootNode;
}

void WidgetView::RelayoutScene()
{
	mpRootNode->DoLayoutChildren( mpScene->GetCanvas() );
}

ResTemplateManager& WidgetView::GetTemplateManager()
{
	WidgetDocument* pDoc = (WidgetDocument*)GetDocument();

	return pDoc->GetTemplateManager();
}

WidgetDocument& WidgetView::GetWidgetDoc()
{
	return *(WidgetDocument*)GetDocument();
}

void WidgetView::SetGridType( int type )
{
	mGridType = type;
}

void WidgetView::GetTopLevelWidgetOrigin( int* x, int* y )
{
	*x = mpRootNode->mBounds.x;
	*y = mpRootNode->mBounds.y;
}

double WidgetView::GetVerticalPixelsPerUnitRatio()
{
	return 1.0;
}

double WidgetView::GetHorizontalPixelsPerUnitRatio()
{
	return 1.0;
}

void WidgetView::PixelRectToUnitRect( XRect& pixelRect, XRect& unitRect )
{
	int ox = 0, oy = 0;
	GetTopLevelWidgetOrigin( &ox, &oy );

	int relX = pixelRect.x - ox;
	int relY = pixelRect.y - oy;

	unitRect.x = PixelsToUnits( pixelRect.x - ox );
	unitRect.y = PixelsToUnits( pixelRect.y - oy );
	unitRect.width  = PixelsToUnits( pixelRect.width  );
	unitRect.height = PixelsToUnits( pixelRect.height );
}

void WidgetView::UnitRectToPixelRect( XRect& unitRect, XRect& pixelRect )
{
	int ox = 0, oy = 0;
	GetTopLevelWidgetOrigin( &ox, &oy );

	pixelRect.x = UnitsToPixels( unitRect.x ) + ox;
	pixelRect.y = UnitsToPixels( unitRect.y ) + oy;

	pixelRect.width  = UnitsToPixels( unitRect.width );
	pixelRect.height = UnitsToPixels( unitRect.height );
}

int WidgetView::AdjustToSnapping( int units )
{
	return ( units / mSnapSpacing.x ) * mSnapSpacing.x;
}

int WidgetView::GetGridType()
{
	return mGridType;
}

wxSize& WidgetView::GetGridSpacinig() { return mGridSpacing; }

wxSize& WidgetView::GetSnapSpacing() { return mSnapSpacing; }

void WidgetView::AdjustRectToGrid( XRect& rect )
{
	int ox = 0, oy = 0;
	GetTopLevelWidgetOrigin( &ox, &oy );

	int relX = rect.x - ox;
	int relY = rect.y - oy;

	rect.x = ox + UnitsToPixels( AdjustToSnapping( PixelsToUnits( relX ) ) );
	rect.y = oy + UnitsToPixels( AdjustToSnapping( PixelsToUnits( relY ) ) );

	rect.width  = UnitsToPixels( AdjustToSnapping( PixelsToUnits( rect.width  ) ) );
	rect.height = UnitsToPixels( AdjustToSnapping( PixelsToUnits( rect.height ) ) );
}

int WidgetView::PixelsToUnits( int pixels )
{
	return pixels;
}

int WidgetView::UnitsToPixels( int units )
{
	return units;
}

void WidgetView::OnActivateView(bool activate, wxView *activeView, 
									wxView *deactiveView)
{
	if ( activate )
	{
		if ( !mpPropPane )
		{
			wxPMView::OnActivateView( activate, activeView, deactiveView );

			wxPMService* pSvc = wxPMBootSvc().FindServiceByClass( 
					CLASSINFO( wxDMNodePropertyPanel ) );

			if ( !pSvc ) return;

			mpPropPane = (wxDMNodePropertyPanel*)pSvc->GetWxObject();

			GetRootNode().FindCmdProcessor()->AddListener( mpPropPane->GetGroupEditor() );
		}

		wxDMNodePropertyGroupEditor* pEditor = mpPropPane->GetGroupEditor();

		pEditor->Update( &GetRootNode() );
	}

	wxPMView::OnActivateView( activate, activeView, deactiveView );
}

void WidgetView::OnUpdate( wxView* sender, wxObject* hint )
{
	mpRootNode = (WidgetNode*)((WidgetDocument*)GetDocument())->GetRootNode();

	if ( !mpRootNode ) return;

	mpXCmdMgr->SetCommandProcessor( mpRootNode->FindCmdProcessor() );

	RecreateScene();

	GetRootNode().FindCmdProcessor()->AddListener( this );
}

void WidgetView::RecreateScene()
{
	mpScene->SetFocus( NULL, FALSE );
	mpScene->ResetSelection();

	mGlyphHash.Clear();

	XObjListT& lst = mpScene->GetContents();

	XObject* pBg = mpScene->GetBackgroundObj();

	mpScene->Dettach( pBg );

	WX_CLEAR_ARRAY( lst );

	mpScene->Insert( pBg );

	CreateGlyphsRecursive( lst, *mpRootNode );

	InvalidateLooks();

	ReorderAllGlyphs();

	RelayoutScene();

	mpScene->RefreshNow();
}

bool WidgetView::OnCreate( wxDocument* doc, long flags )
{
	if ( !wxPMView::OnCreate( doc, flags ) ) return FALSE;

	wxASSERT( doc->IsKindOf( CLASSINFO(WidgetDocument) ) ); // DBG::

	mpScene = new MyXCollection( this );

	mpScene->SetCanvasOrigin( XPoint(30,30) );

	XCellBackground* pBg = new XCellBackground( *wxWHITE );
	mpScene->Insert( pBg );

	XSelectionEditor* pEditor = new XSelectionEditor();
	mpScene->SetEditor( pEditor );

	mpScene->MarkBackgroundObj( *pBg );

	mpXCmdMgr = new MyXCommandMgr( NULL, mpScene, this );
	
	XwxWindowHook* pHook = new XwxWindowHook( GetViewFrame(), mpScene );

	mpEditorHandler->SetLayoutUtility( new XLayoutUtility() );
	mpEditorHandler->SetCurrentScene( *mpScene );
	mpEditorHandler->SetCurrentCmdManager( *mpXCmdMgr );

	return TRUE;
}

wxPMMergableMenuChain* WidgetView::GetMergableMenuChain()
{
	if ( !mpMenuChains )

		RecreateMenuChains();

	return mpMenuChains;
}

void WidgetView::OnEditorCommandsRange( wxCommandEvent& event )
{
	mpEditorHandler->ProcessEvent( event );
}

void WidgetView::OnInsertWidgetCommandsRange( wxCommandEvent& event )
{
	size_t widgetNo = event.m_id - ID_INSERT_WIDGET_IDS_START;


	WidgetInsetionEditor* pEditor = new WidgetInsetionEditor( this, widgetNo );

	/*
	XPoint pos; WidgetNode* pTarget = NULL;

	GetWidgetDoc().FindInsertionPoint( pos, pTarget );
	GetWidgetDoc().InsertWidgetByCode( widgetNo, pTarget, pos );

	mpScene->RefreshNow();
	*/
}

void WidgetView::OnSetUIType( wxCommandEvent& event )
{
	wxString choices[6] = { "WINDOWS", "GTK", "PM", "MOTIF", "QT", "MAC" };

	wxSingleChoiceDialog dlg( wxPMBootSvc().GetMainWindow(), "Types:", 
							  "Set UI-Type", 6, choices );

	wxString type = GetWidgetDoc().GetUIType();

	for( size_t i = 0; i != 6; ++i )

		if ( choices[i] == type ) 
		{
			dlg.SetSelection( i ); break;
		}

	if ( dlg.ShowModal() == wxID_OK )
	{
		GetWidgetDoc().SetUIType( dlg.GetStringSelection() );

		RecreateScene();
	}
}

void WidgetView::OnSetUnits( wxCommandEvent& event )
{
	wxString choices[2] = { "Dialog Units", "Pixels" };

	wxSingleChoiceDialog dlg( wxPMBootSvc().GetMainWindow(), "Unit types:", 
							  "Set Unit", 2, choices );

	if ( dlg.ShowModal() == wxID_OK )
	{
		mUseDialogUnits = dlg.GetStringSelection() == "Dialog Units";

		RecreateScene();
	}
}

void WidgetView::OnSetSizer( wxCommandEvent& event )
{
	wxMessageBox( "FOR NOW:: use menu \"Insert | wxBoxSizer\"" );
}

void WidgetView::OnGridSettings( wxCommandEvent& event )
{
	GridSettingsDialog dlg( wxPMBootSvc().GetMainWindow(), 
							mGridType, mGridSpacing, mSnapSpacing );

	if ( dlg.ShowModal() == wxID_OK )
	{
		mGridType = dlg.GetGridType();
		mGridSpacing = dlg.GetGridSpacinig();
		mSnapSpacing = dlg.GetSnapSpacing();

		RefreshScene();
	}
}

void WidgetView::OnInsertWidget( wxCommandEvent& event )
{
	wxMessageBox( "OnInsertWidget" );
}

void WidgetView::OnPreview( wxCommandEvent& event )
{
	wxMessageBox( "OnPreview" );
}

void WidgetView::OnReloadWigetLooks( wxCommandEvent& event )
{
	wxMessageBox( "OnReloadWigetLooks" );
}

void WidgetView::OnShowSizers( wxCommandEvent& event )
{
	wxMenuBar* pBar = wxPMBootSvc().GetMainWindow()->GetMenuBar() ;

	// m_callbackUserData holds the orignal item-id (not affected
	// by PMF's transaltions

	if ( pBar->IsChecked( event.m_commandInt ) )
	
		mSizersShown = TRUE;
	else
		mSizersShown = FALSE;

	mpScene->InvalidateRect( GetGlyphForNode( *mpRootNode ).GetBounds() );

	mpScene->RefreshNow();
}

bool WidgetView::SizersShown()
{
	return mSizersShown;
}

/*** notifications ***/

void WidgetView::OnNodeRemoved( wxDMNode& node )
{
	for( size_t i = 0; i != node.GetChildCount(); ++i )

		OnNodeRemoved( *node.GetChild(i) );

	GetWidgetDoc().MarkUnusedId( ((WidgetNode*)&node)->GetNodeAttrValue( "name" ) );

	WidgetGlyph& glyph = GetGlyphForNode( node );

	mGlyphHash.Delete( (long)&node );

	WidgetNode* pParent = (WidgetNode*)node.GetParent();

	mpScene->Remove( &glyph, FALSE );

	if ( pParent && pParent->IsKindOf( CLASSINFO( SizerNodeBase ) ) )

		RelayoutScene();
}

void WidgetView::OnNodeAdded( wxDMNode& node )
{
	GetWidgetDoc().MarkUsedId( ((WidgetNode*)&node)->GetNodeAttrValue( "name" ) );

	WidgetGlyph* pGlyph = CreateGlyphForNode( *(WidgetNode*)&node );

	mpScene->Insert( pGlyph, FALSE );

	for( size_t i = 0; i != node.GetChildCount(); ++i )

		OnNodeAdded( *node.GetChild(i) );

	WidgetNode* pParent = (WidgetNode*)node.GetParent();

	if ( pParent && pParent->IsKindOf( CLASSINFO( SizerNodeBase ) ) )

		RelayoutScene();
}

void WidgetView::OnNodeChanged( wxDMNode& node )
{
	// if the modification came from external party
	// (e.g. property editor) then reflect changes to
	// node as well as it's to view

	if ( !GetWidgetDoc().GetCmdProcessor().GetChangeSender()->
			IsKindOf( CLASSINFO( XObject ) ) )
	{
		WidgetNode& wnode = *((WidgetNode*)&node);

		wnode.ExtractBoundsInfo();

		wnode.SetBounds( wnode.GetBounds(), mpScene->GetCanvas() );

		WidgetGlyph& glyph = GetGlyphForNode( node );

		glyph.InvalidateLook();

		mpScene->InvalidateRect( glyph.GetInclusiveBounds() );

		AssignPropertiesToGlyph( glyph, *(WidgetNode*)&node );

		WidgetNode* pParent = (WidgetNode*)node.GetParent();

		pParent->OnChildChanged( &node );

		// FIMXE:: dirty
		if ( pParent && pParent->IsKindOf( CLASSINFO( SizerNodeBase ) ) )

			RelayoutScene();

		mpScene->InvalidateRect( glyph.GetInclusiveBounds() );

		glyph.GetEditor().NotifyObjectChanged();

		mpScene->RefreshNow();
	}
}

void WidgetView::OnNodeOrderChanged( wxDMNode& node )
{
	ReorderAllGlyphs();
}

void WidgetView::OnPointerChanged( wxDMNode* pFromPtr, wxDMNode* pToPtr )
{
}

void WidgetView::OnNodeReplaced( wxDMNode& node, wxDMNode& withNode )
{
	WidgetGlyph& glyph = GetGlyphForNode( *(WidgetNode*)&node );

	mpScene->Remove( &glyph, FALSE );

	mGlyphHash.Delete( (long)&node );

	WidgetGlyph* pGlyph = CreateGlyphForNode( *(WidgetNode*)&withNode );

	mpScene->Insert( pGlyph, FALSE );

	if ( &node == mpRootNode )
	{
		mpRootNode = (WidgetNode*)&withNode;
		GetWidgetDoc().SetRootNode( mpRootNode );
	}

	ReorderAllGlyphs();

	RelayoutScene();
}

void WidgetView::OnFocusChanged( wxDMNode& node )
{
	mpScene->SetFocus( &GetGlyphForNode( node ) );
}

void WidgetView::OnFocusRemoved()
{
	mpScene->SetFocus( NULL );
}

void WidgetView::OnSelectionSet( DMNodeListT& nodes )
{
	XObjListT lst;

	for( size_t i = 0; i != nodes.Count(); ++i )

		lst.Add( &GetGlyphForNode( *nodes[i] ) );

	mpScene->SetSelection( lst );
}

void WidgetView::OnNodeAddedToSelection( wxDMNode& node )
{
	// TBD:: ?
}

void WidgetView::OnNodeDeselected( wxDMNode& node )
{
	// TBD:: ?
}

void WidgetView::OnUpdateFinished()
{
	if ( mpScene->HasEditor() ) mpScene->GetEditor().NotifyObjectChanged();

	mpScene->RefreshNow();
}

/*** protected methods ***/

// helpers
XRect wxRectToXRect( const wxRect& rect )
{
	return XRect( rect.x, rect.y, rect.width, rect.height );
}

wxRect XRectToWxRect( const XRect& rect )
{
	return wxRect( rect.x, rect.y, rect.width, rect.height );
}

void WidgetView::AssignPropertiesToGlyph( WidgetGlyph& glyph, WidgetNode& fromNode )
{
	XRect bounds = 	fromNode.GetBounds();

	glyph.ToGlyphRect( bounds );

	glyph.SetBoundsSilent( bounds );
}

WidgetGlyph* WidgetView::CreateGlyphForNode( WidgetNode& node )
{
	wxClassInfo* pClass = 
		wxClassInfo::FindClass( (char*)(const char*)node.mpTempl->mPreviewerClass );

	WidgetGlyph* pGlyph = NULL;

	if ( pClass ) pGlyph = (WidgetGlyph*)pClass->CreateObject();

	if ( !( pClass && pGlyph ) )
	{
		wxMessageBox( "Cannot create previewer object of class " + 
					  node.mpTempl->mPreviewerClass );

		pGlyph = new WidgetGlyph();
	}

	pGlyph->SetNode( &node );

	pGlyph->SetParent( mpScene );

	mGlyphHash.Put( (long)&node, pGlyph );

	AssignPropertiesToGlyph( *pGlyph, node );

	XRectEditor* pEditor = new XRectEditor();
	pEditor->SetAutoRise( FALSE );

	if ( &node == mpRootNode )

		pEditor->EnableMove( FALSE );

	pGlyph->SetEditor( pEditor );

	return pGlyph;
}

void WidgetView::AddGlyphRefsRecursive( XObjListT& toLst, wxDMNode& node )
{
	WidgetGlyph* pGlyph = FindGlyphForNode( node );

	if ( pGlyph )
	{
		mpScene->InvalidateRect( pGlyph->GetInclusiveBounds() );

		toLst.Add( pGlyph );
	}

	for( size_t i = 0; i != node.GetChildCount(); ++i )

		AddGlyphRefsRecursive( toLst, *node.GetChild(i) );
}

void WidgetView::CreateGlyphsRecursive( XObjListT& toLst, wxDMNode& node )
{
	WidgetGlyph* pGlyph = CreateGlyphForNode( *(WidgetNode*)&node );

	for( size_t i = 0; i != node.GetChildCount(); ++i )

		CreateGlyphsRecursive( toLst, *node.GetChild( i ) );

	pGlyph->GetNode().RecalcPreferredSize( mpScene->GetCanvas() );

	mpScene->Insert( pGlyph, FALSE );
}

void WidgetView::ReorderAllGlyphs()
{
	XObjListT& lst = mpScene->GetContents();

	XObject* pBg = mpScene->GetBackgroundObj();

	lst.Clear();

	lst.Add( pBg );

	AddGlyphRefsRecursive( lst, *mpRootNode );
}

void WidgetView::RefreshScene()
{
	XObjListT& lst = mpScene->GetContents();

	XObject* pBg = mpScene->GetBackgroundObj();

	for( size_t i = 0; i != lst.Count(); ++i )

		if( lst[i] != pBg )

			mpScene->InvalidateRect( lst[i]->GetInclusiveBounds() );

	mpScene->RefreshNow();
}

void WidgetView::InvalidateLooks()
{
	XObject* pBg = mpScene->GetBackgroundObj();

	XObjListT& lst = mpScene->GetContents();

	for( size_t i = 0; i != lst.Count(); ++i )
	{	
		XObject* pObj = lst[i];

		if ( pObj != pBg )

			((WidgetGlyph*)pObj)->InvalidateLook();
	}
}

void WidgetView::RecreateMenuChains()
{
	if ( mpWidgetMenus ) delete [] mpWidgetMenus;
	if ( mpMenuChains ) delete [] mpMenuChains;

	wxPMMergableMenuChain* pEditorChains = mpEditorHandler->GetMergableMenuChain();
	wxPMMergableMenuChain* pChain = pEditorChains;

	size_t sz = 0;
	while( pChain->m_pCommands )
	{
		++sz;
		++pChain;
	}

	// prepare items for the "Insert" menu

	WidgetResTemplateListT& lst = GetTemplateManager().GetTemplates();

	mpWidgetMenus = new wxCommandInfo[ lst.Count() + 1 ];

	int id = ID_INSERT_WIDGET_IDS_START;

	size_t i = 0;
	for( ; i != lst.Count(); ++i )
	{
		wxCommandInfo& cmd = mpWidgetMenus[i];
		cmd.SetTitle( lst[i]->mWidgetClass );
		cmd.SetId( id++ );
		cmd.SetHelpString( "Insert " + lst[i]->mWidgetClass );

		if ( lst[i]->mToolBarBitmap.Ok() )
		{
			wxBitmap bmp = lst[i]->mToolBarBitmap;

			wxMask* pMask = new wxMask();
			pMask->Create( bmp, *wxRED );
			bmp.SetMask( pMask );

			cmd.SetBitmap( bmp );
		}

		//cmd.SetBitmap( wxPMBmpMgr().FindBitmap( "filetypes/16x16_wrs_type" ) );
	}

	mpWidgetMenus[ lst.Count() ].SetId(0); // terminator
	mpWidgetMenus[ lst.Count() ].SetTitle("");

	// put all tougeahter into chanes

	mpMenuChains = new wxPMMergableMenuChain[ sz + 3 ];

	for( i = 0; i != sz; ++i )

		mpMenuChains[i] = pEditorChains[i];

	mpMenuChains[sz+0].m_PredcessorPath = "/&Canvas/";
	mpMenuChains[sz+0].m_pCommands = _gCanvasCommands;
	mpMenuChains[sz+1].m_PredcessorPath = "/&Insert/";
	mpMenuChains[sz+1].m_pCommands = mpWidgetMenus;
	mpMenuChains[sz+2].m_pCommands = NULL; // terminator

}

/***** Implementation for class WidgetEqSolver *****/

WidgetEqSolver::WidgetEqSolver()

	: mpGroup( NULL ),
	  mpGroupData( NULL ),
	  mCurVarIdx( 0 )
{}

void WidgetEqSolver::Init( wxDMAttrGroupInfo* pGroup, 
						   void* pGroupData,
						   const wxString& uiType )
{
	mpGroup = pGroup;
	mpGroupData = pGroupData;
	mUIType = uiType;
}

EqSolverVar* WidgetEqSolver::FindVar( const wxString& name )
{
	if ( name == "widgetclass" )

		return GetVarForValue( mpGroup->GetName() );
	else
	if ( name == "UI_TYPE" )

		return GetVarForValue( mUIType );
	else
	{
		wxDMAttrInfo* pAttr = mpGroup->GetAttrByName( name );

		if ( !pAttr )

			return EqSolver::FindVar( name );

		return GetVarForValue( pAttr->GetStringValue( mpGroupData ) );
	}
}

/*** protected methods ***/

EqSolverVar* WidgetEqSolver::GetVarForValue( const wxString& value )
{
	++mCurVarIdx;

	if ( mCurVarIdx == 5 ) mCurVarIdx = 0;

	mVarsRibbon[ mCurVarIdx ].SetString( value );

	return &mVarsRibbon[ mCurVarIdx ];
}

/***** Implementation for class WidgetNode *****/

IMPLEMENT_DYNAMIC_CLASS( WidgetNode, wxDMCompositeNode )

WidgetNode::WidgetNode()

	: mBgColor( *wxRED ),
	  mpTempl( NULL ),
	  mpView( NULL )
{}

WidgetNode::~WidgetNode()
{}

size_t WidgetNode::GetGroupCount()
{
	return HasExtraProperties() ? 2 : 1;
}

wxDMAttrGroupInfo* WidgetNode::GetAttrGroupInfo( size_t index )
{
	return mAttribGroups[index].GetGroupInfo();
}

void* WidgetNode::GetAttrGroupData( size_t index )
{
	return mAttribGroups[index].GetData();
}

void* WidgetNode::GetAttrGroupData( wxDMAttrGroupInfo* pForGroup )
{
	for( size_t i = 0; i != MAX_PROPERTY_GROUPS; ++i )

		if ( mAttribGroups[i].GetGroupInfo() == pForGroup )

			return &mAttribGroups[i];

	return NULL;
}

wxString WidgetNode::GetDisplayName()
{
	return GetNodeAttrValue( "name" );
}

void WidgetNode::SetView( WidgetView* pView )
{
	mpView = pView;
}

const wxString& WidgetNode::GetWidgetClass()
{
	return mpTempl->mWidgetClass;
}

XRect& WidgetNode::GetBounds()
{
	return mBounds;
}

void WidgetNode::SetBounds( const XRect& rect, wxDC& dc )
{
	mBounds = rect;

	WidgetGlyph* pGlyph = mpView->FindGlyphForNode( *this );

	mpView->AdjustRectToGrid( mBounds );

	StoreBoundsInfo();

	if ( pGlyph )
	{
		XRect bounds = mBounds;

		pGlyph->ToGlyphRect( bounds );

		pGlyph->SetBoundsSilent( bounds );
	}

	DoLayoutChildren( dc );
}

void WidgetNode::SetBgColor( const wxColour& col )
{
	mBgColor = col;
}

wxColour& WidgetNode::GetBgColor()
{
	return mBgColor;
}

bool WidgetNode::HasExtraProperties()
{
	return mAttribGroups[EXTRA_PROPERTIES_GROUP].mAttrData.Count() != 0;
}

void WidgetNode::ExtractBoundsInfo()
{
	// FIXME:: this can be slow..

	void* pData = GetAttrGroupData( WIDGET_PROPERTIES_GROUP );
	wxDMAttrGroupInfo* pGroup = mAttribGroups[WIDGET_PROPERTIES_GROUP].GetGroupInfo();

	mBounds.x = StrToInt( pGroup->GetAttrValueByName( "x", pData ) );
	mBounds.y = StrToInt( pGroup->GetAttrValueByName( "y", pData ) );
	mBounds.width  = StrToInt( pGroup->GetAttrValueByName( "width",  pData ) );
	mBounds.height = StrToInt( pGroup->GetAttrValueByName( "height", pData ) );

	if ( mBounds.x == -1 ) mBounds.x = 0;
	if ( mBounds.y == -1  ) mBounds.y = 0;
}

void WidgetNode::StoreBoundsInfo()
{
	// FIXME:: this can be slow..

	void* pData = GetAttrGroupData( WIDGET_PROPERTIES_GROUP );
	wxDMAttrGroupInfo* pGroup = mAttribGroups[WIDGET_PROPERTIES_GROUP].GetGroupInfo();

	pGroup->SetAttrValueByName( "x", IntToStr( mBounds.x ), pData );
	pGroup->SetAttrValueByName( "y", IntToStr( mBounds.y ), pData );
	pGroup->SetAttrValueByName( "width",  IntToStr( mBounds.width  ), pData );
	pGroup->SetAttrValueByName( "height", IntToStr( mBounds.height ), pData );
}

wxString WidgetNode::GetNodeAttrValue( const wxString& name )
{
	ResGroupData& grpData = mAttribGroups[WIDGET_PROPERTIES_GROUP];

	return grpData.mpGroupInfo->GetAttrValueByName( name, grpData.GetData() );
}

wxString WidgetNode::GetNodeExtraAttrValue( const wxString& name )
{
	ResGroupData& grpData = mAttribGroups[EXTRA_PROPERTIES_GROUP];

	return grpData.mpGroupInfo->GetAttrValueByName( name, grpData.GetData() );
}

void WidgetNode::SetNodeAttrValue( const wxString& name, const wxString& value )
{
	ResGroupData& grpData = mAttribGroups[WIDGET_PROPERTIES_GROUP];

	grpData.mpGroupInfo->SetAttrValueByName( name, value, grpData.GetData() );
}

void WidgetNode::SetNodeExtraAttrValue( const wxString& name, const wxString& value )
{
	ResGroupData& grpData = mAttribGroups[EXTRA_PROPERTIES_GROUP];

	grpData.mpGroupInfo->SetAttrValueByName( name, value, grpData.GetData() );
}

void WidgetNode::OnChildInserted( wxDMNode* pNode )
{
	if ( mpTempl->mpExtraAttrGroup )

		AddExtraPropertiesToChild( (WidgetNode*)pNode );
}

void WidgetNode::OnChildRemoved( wxDMNode* pNode )
{
	WidgetNode* pWNode = (WidgetNode*)pNode;

	if ( pWNode->HasExtraProperties() )

		pWNode->mAttribGroups[ EXTRA_PROPERTIES_GROUP ].CleanUp();
}

XRect WidgetNode::GetLimitingBounds()
{
	WidgetGlyph* pGlyph = mpView->FindGlyphForNode( *this );

	if ( !pGlyph ) return XRect();

	// FIXME:: dirty hack!!!

	XRect bounds = pGlyph->GetBounds();

	XRect lbounds = pGlyph->GetLimitingBounds();

	lbounds.x -= bounds.x;
	lbounds.y -= bounds.y;

	return lbounds;
}

void WidgetNode::DoLayoutChildren( wxDC& dc )
{
	// TBD:: comments...

	if ( mChildren.Count() == 1 &&
		 mChildren[0]->IsKindOf( CLASSINFO( SizerNodeBase ) ) )
	{
		XRect lbounds = GetLimitingBounds();

		WidgetNode* pChild = (WidgetNode*)mChildren[0];

		pChild->SetBounds( lbounds, dc );

		return;
	}

	for( size_t i = 0; i != mChildren.Count(); ++i )
	{
		WidgetNode& node = *(WidgetNode*)mChildren[i];

		XRect bounds = node.GetBounds();

		node.SetBounds( bounds, dc );
	}
}

void WidgetNode::AddExtraPropertiesToChild( WidgetNode* pChild )
{
	if ( this->mpTempl->mpExtraAttrGroup )
	{
		if ( pChild->mAttribGroups[ EXTRA_PROPERTIES_GROUP ].mpGroupInfo 
			 != this->mpTempl->mpExtraAttrGroup )
		{
			pChild->mAttribGroups[ EXTRA_PROPERTIES_GROUP ].CleanUp();

			pChild->mAttribGroups[ WidgetNode::EXTRA_PROPERTIES_GROUP ].
				SetGroupInfo( this->mpTempl->mpExtraAttrGroup );
		}
	}
	else
	if ( pChild->HasExtraProperties() )

		pChild->mAttribGroups[ EXTRA_PROPERTIES_GROUP ].CleanUp();
}

wxDMNode* WidgetNode::Clone( bool uniqueCopy, bool cloneChildren )
{
	wxDMNode* pCopy = new WidgetNode();

	AssignData( pCopy, uniqueCopy, cloneChildren );

	return pCopy;
}

void WidgetNode::AssignData( wxDMNode* pCopy, bool uniqueCopy, bool cloneChildren )
{
	WidgetNode* pNode = (WidgetNode*)pCopy;

	pNode->mBgColor = mBgColor;
	pNode->mBounds = mBounds;
	pNode->mpTempl = mpTempl;
	pNode->mpView = mpView;

	for( size_t i = 0; i != MAX_PROPERTY_GROUPS; ++i )

		mAttribGroups[i].CopyDataTo( pNode->mAttribGroups[i] );

	if ( uniqueCopy )
	{
		wxString newId = mpView->GetWidgetDoc().GenerateUniqueId( *pNode );

		pNode->SetNodeAttrValue( "name", newId );
	}

	wxDMCompositeNode::AssignData( pCopy, uniqueCopy, cloneChildren );
}

WidgetResTemplate& WidgetNode::GetTemplate()
{
	return *mpTempl;
}

const XDim WidgetNode::GetPreferredSize()
{
	return mPrefSize;
}

void WidgetNode::SetPreferredSize( const XDim& size )
{
	mPrefSize;
}

void WidgetNode::OnStartEdit()
{
	NotifyWillChange( this );
}

void WidgetNode::OnFinishEdit()
{
	NOTIFY_ONE_ARG( OnNodeChanged, *this );
}

void WidgetNode::RecalcPreferredSize( wxDC& dc )
{
	// FIXME:: dirty (should not glyph for that matter)

	WidgetGlyph* pGlyph = mpView->FindGlyphForNode( *this );

	if ( !pGlyph ) return;

	wxUILookInfo* pInfo = pGlyph->GetLookInfo();

	if ( pInfo == NULL )
	{
		mPrefSize.width  = 20;
		mPrefSize.height = 20;

		return;
	}

	wxString text;
	int textWidth = 0, textHeight = 0,
		textX = 0, textY = 0,
		itemWidth = 0, itemHeight = 0;

	WidgetDocument& doc = mpView->GetWidgetDoc();
	WidgetEqSolver& solver = doc.GetEqSolver();

	solver.Init( GetAttrGroupInfo( WidgetNode::WIDGET_PROPERTIES_GROUP ),
				 GetAttrGroupData( WidgetNode::WIDGET_PROPERTIES_GROUP ),
				 mpView->GetWidgetDoc().GetUIType() );
	
	doc.GetRenderer().CalcItemParams( *pInfo, 
									  wxSize(10,10), // doesn't matter here
									  dc, text, 
									  textWidth, textHeight, 
									  textX, textY, itemWidth, itemHeight, &solver );

	mPrefSize.width  = itemWidth;
	mPrefSize.height = itemHeight;
}

/***** Implementation for class WidgetGlyph *****/

IMPLEMENT_DYNAMIC_CLASS( WidgetGlyph, XObject )

WidgetGlyph::WidgetGlyph()

	: mpLookInfo( NULL ),
	  mpBmp( NULL ),
	  mpNode( NULL )
{}

WidgetGlyph::~WidgetGlyph()
{
	if ( mpBmp ) delete mpBmp;
}

void WidgetGlyph::SetNode( WidgetNode* pNode )
{
	mpNode = pNode;
}

void WidgetGlyph::SetLookInfo( wxUILookInfo* pInfo )
{
	mpLookInfo = pInfo;
}

wxUILookInfo* WidgetGlyph::GetLookInfo()
{
	if ( !mpLookInfo )

		mpLookInfo = mpNode->mpView->GetWidgetDoc().FindLookForNode( *mpNode );

	return mpLookInfo;
}

bool WidgetGlyph::Contains( const XPoint& point )
{
	if ( mpLookInfo && mpLookInfo->mIsOpaque == FALSE )
	{
		XRect r = mPixBounds;

		r.x      += mpLookInfo->mEdges[ LEFT_LOOK_EDGE ].mWidth;
		r.width  -= mpLookInfo->mEdges[ LEFT_LOOK_EDGE ].mWidth + 
					mpLookInfo->mEdges[ RIGHT_LOOK_EDGE ].mWidth;

		r.y      += mpLookInfo->mEdges[ TOP_LOOK_EDGE ].mWidth;
		r.height -= mpLookInfo->mEdges[ TOP_LOOK_EDGE ].mWidth + 
					mpLookInfo->mEdges[ BOTTOM_LOOK_EDGE ].mWidth;

		// contains only if the inner (transparent) area is not hit

		return !r.Contains( point );
	}
	else
		return TRUE; // for opaque components, every point
					 // of their interiour is "insider"
}

void WidgetGlyph::Draw( wxDC& canvas )
{
	if ( !mpBmp )

		RegenerateLookBitmap();

	if ( !mpBmp )
	{
		wxBrush br( mpNode->GetBgColor(), wxSOLID );

		wxPen pen( *wxWHITE, 2, wxSOLID );
		canvas.SetPen( pen );

		canvas.SetBrush( br );

		canvas.DrawRectangle( mPixBounds.x, mPixBounds.y, mPixBounds.width, mPixBounds.height );

		canvas.DrawLine( mPixBounds.x, mPixBounds.y, 
						 mPixBounds.x + mPixBounds.width - 1, mPixBounds.y + mPixBounds.height - 1 );

		canvas.DrawLine( mPixBounds.x, mPixBounds.y + mPixBounds.height - 1, 
						 mPixBounds.x + mPixBounds.width - 1, mPixBounds.y );

		canvas.SetTextForeground( *wxBLUE );
		canvas.DrawText( mpNode->mpTempl->mWidgetClass, mPixBounds.x + 5, mPixBounds.y + 5 );
	}
	else
	
		mpNode->mpView->GetWidgetDoc().GetRenderer().
			DrawLook( canvas, *this->mpLookInfo, *mpBmp, mPixBounds.x, mPixBounds.y );

		//canvas.DrawBitmap( *mpBmp, mPixBounds.x, mPixBounds.y );

	DrawGrid( canvas );
}

void WidgetGlyph::DrawGrid( wxDC& dc )
{
	if ( !mpLookInfo ) return;

	if ( !mpNode->mpTempl->mCanHaveChildren ) return;

	int type = mpNode->mpView->GetGridType();

	if ( type == WidgetView::GRID_NONE ) return;

	XRect inner;
	GetLimitingRectInUnits( inner );

	int stepx = mpNode->mpView->GetGridSpacinig().x;
	int stepy = mpNode->mpView->GetGridSpacinig().x;

	int fromx = ( (inner.x  ) / stepx ) * stepx;
	int fromy = ( ( inner.y  ) / stepy ) * stepy;
	int tillx = ( ( inner.x + inner.width  ) / stepx ) * stepx;
	int tilly = ( ( inner.y + inner.height ) / stepy ) * stepy;
	if ( fromx < inner.x ) fromx += stepx;
	if ( fromy < inner.y ) fromy += stepy;
	if ( tillx >= inner.x + inner.width ) tillx -= stepx;
	if ( tilly >= inner.y + inner.height) tilly -= stepy;

	int ox = 0, oy = 0;
	mpNode->mpView->GetTopLevelWidgetOrigin( &ox, &oy );

	double pixPerUnitV = mpNode->mpView->GetVerticalPixelsPerUnitRatio(), 
		   pixPerUnitH = mpNode->mpView->GetHorizontalPixelsPerUnitRatio();

	if ( type == WidgetView::GRID_LINES )
	{
		int left   = inner.x * pixPerUnitH + ox;
		int right  = ( inner.x + inner.width ) * pixPerUnitH + ox;
		int top    = inner.y * pixPerUnitV + oy;
		int bottom = ( inner.y + inner.height ) * pixPerUnitV + oy;

		dc.SetPen( *wxGREY_PEN );

		dc.DrawLine( left, top, right, top );
		dc.DrawLine( right, top, right, bottom );
		dc.DrawLine( right, bottom, left, bottom );
		dc.DrawLine( left, bottom, left, top );

		for( int y = fromy; y <= tilly; y += stepy )
		{
			int pixely = oy + y * pixPerUnitV;

			dc.DrawLine( left, pixely, right, pixely );
		}

		for( int x = fromx; x <= tillx; x += stepx )
		{
			int pixelx = ox + x * pixPerUnitH;

			dc.DrawLine( pixelx, top, pixelx, bottom );
		}
	}
	else
	{
		if ( type == WidgetView::GRID_DOTS_3D )
			
			dc.SetPen( *wxBLACK_PEN );
		else
			dc.SetPen( *wxGREY_PEN );

		{
			for( int y = fromy; y <= tilly; y += stepy )
			{
				int pixely = oy + y * pixPerUnitV;

				for( int x = fromx; x <= tillx; x += stepx )
				{
					int pixelx = ox + x * pixPerUnitH;

					dc.DrawPoint( pixelx, pixely );
				}
			}
		}

		if ( type == WidgetView::GRID_DOTS_3D )
		{
			dc.SetPen( *wxWHITE_PEN );

			for( int y = fromy; y <= tilly; y += stepy )
			{
				int pixely = oy + y * pixPerUnitV + 1;

				for( int x = fromx; x <= tillx; x += stepx )
				{
					int pixelx = ox + x * pixPerUnitH + 1;

					dc.DrawPoint( pixelx, pixely );
				}
			}
		}
	}
}

void WidgetGlyph::SetBoundsSilent( const XRect& bounds )
{
	if ( mpEditor )

		InvalidateRect( mpEditor->GetInclusiveBounds() );

	int oldWidth = mPixBounds.width, oldHeight = mPixBounds.height;

	mPixBounds = bounds;

	if ( mpBmp && 
		 ( oldWidth  != mPixBounds.width ||
		   oldHeight != mPixBounds.height ) ) 
	{
		delete mpBmp;
		mpBmp = NULL; // invalidate current look-image
	}

	InvalidateRect( GetBounds() );
}

void WidgetGlyph::SetBounds( const XRect& bounds )
{
	int oldWidth = mPixBounds.width, oldHeight = mPixBounds.height;

	mPixBounds = bounds;

	mpNode->mpView->AdjustRectToGrid( mPixBounds );

	if ( mpBmp && 
		 ( oldWidth  != mPixBounds.width ||
		   oldHeight != mPixBounds.height ) ) 
	{
		delete mpBmp;
		mpBmp = NULL; // invalidate current look-image
	}

	// FIXME::

	XRect nodeBounds = bounds;
	ToNodeRect( nodeBounds );

	mpNode->SetBounds( nodeBounds, GetCanvas() );

	// apply layouting of children which was done by setting bounds to 
	// non-visual node

	// FIXME:: BS!

#if 0
	for( size_t i = 0; i != mpNode->GetChildCount(); ++i )
	{
		WidgetNode& child = *(WidgetNode*)mpNode->GetChild(i);

		XRect newBounds = child.GetBounds();

		WidgetGlyph* pGlyph = mpNode->mpView->FindGlyphForNode( child );

		if ( pGlyph )
		{
			pGlyph->ToGlyphRect( newBounds );

			InvalidateRect( pGlyph->GetBounds() );
			pGlyph->SetBounds( newBounds );
		}

		InvalidateRect( newBounds );
	}
#endif
}

XRect WidgetGlyph::GetBounds()
{
	return mPixBounds;
}

XRect WidgetGlyph::GetLimitingBounds()
{
	if ( mpLookInfo )
	{
		// insets subsctracted in units

		XRect r;
		GetLimitingRectInUnits( r );

		XRect lbounds;
		mpNode->mpView->UnitRectToPixelRect( r, lbounds );

		return lbounds;
	}

	return mPixBounds;
}

bool WidgetGlyph::OnStartEdit()
{
	mpNode->OnStartEdit();

	return true;
}

bool WidgetGlyph::OnFinishEdit()
{
	WidgetNode* pParent = (WidgetNode*)mpNode->GetParent();

	if ( pParent )
	{
		WidgetNode* pTarget = NULL; int zOrder = 0;

		mpNode->mpView->FindInsertionTarget( XPoint( mPixBounds.x, mPixBounds.y ), pTarget, 
											 zOrder, mpNode );

		if ( pTarget 
			 &&
			 ( !(pTarget == pParent && zOrder == -1) || // either we are changing item's order
			    pTarget != pParent						// or reparenting
			  ) 
			 && 
			   // guard against insertion into one of own children
			   !mpNode->Contains( pTarget ) 
		   )

		{
			// user has dragged item out of it's parent container meaning
			// to change item's parent or he/she intends to change the order 
			// of children within the parent

			SetEditor( NULL );

			InvalidateRect( GetParent().GetInclusiveBounds() );

			// FIXME:: kludge..
			size_t order = mpNode->GetOrderNo();
			mpNode->GetParent()->GetChildren().Remove( mpNode );

			// FIXME:: kludge..
			mpNode->GetParent()->GetChildren().Insert( mpNode, order );

			WidgetNode* pCopy = (WidgetNode*)mpNode->Clone( FALSE );

			GetWindow()->ReleaseMouse();
			ResetMouseFocus( this );

			XRect bounds = this->GetBounds();

			WidgetGlyph& targetGlyph = pTarget->mpView->GetGlyphForNode( *pTarget );

			// translate new bounds into target-item's local coordinates

			targetGlyph.ToNodeRect( bounds );

			pCopy->SetBounds( bounds, targetGlyph.GetCanvas() );

			if ( pTarget == pParent && zOrder > (int)mpNode->GetOrderNo() )

				--zOrder; // we're removing ourselves, the index should slide left

			XObject& parentObj = GetParent();

			pTarget->FindCmdProcessor()->StartBatch();

			mpNode->NotifyWillRemove( mpNode );
			mpNode->Remove();

			pTarget->AddChild( pCopy, zOrder );

			pCopy->NotifyAdded( pCopy );

			pTarget->FindCmdProcessor()->FinishBatch();

			pTarget->mpView->RelayoutScene();

			parentObj.RefreshNow();

			return FALSE;
		}
	}

	mpNode->FindCmdProcessor()->SetChangeSender( this );

	mpNode->OnFinishEdit();

	if ( pParent && pParent->IsKindOf( CLASSINFO( SizerNodeBase ) ) )
	{	
		mpNode->mpView->RelayoutScene();

		if ( HasEditor() )

			GetEditor().NotifyObjectChanged();
	}

	return TRUE;
}

WidgetGlyph* WidgetGlyph::GetParentGlyph()
{
	WidgetNode* pParent = (WidgetNode*)mpNode->GetParent();

	if( pParent == NULL ) return NULL;

	WidgetGlyph& parentGlyph = mpNode->mpView->GetGlyphForNode( *pParent );

	return &parentGlyph;
}

void WidgetGlyph::ToGlyphRect( XRect& rect )
{
	// rect is given in node's coordinates

	WidgetGlyph* pParent = GetParentGlyph();

	if ( !pParent ) return;

	XRect bounds = pParent->GetBounds();

	// convert them to canvas-coordinates of this glyph

	rect.x += bounds.x;
	rect.y += bounds.y;
}

void WidgetGlyph::ToNodeRect( XRect& rect )
{
	// rect is given in glyph's coordinates

	WidgetGlyph* pParent = GetParentGlyph();

	if ( !pParent ) return;

	XRect bounds = pParent->GetBounds();

	rect.x -= bounds.x;
	rect.y -= bounds.y;
}

WidgetNode& WidgetGlyph::GetNode()
{
	return *mpNode;
}

XObject* WidgetGlyph::GetSurroundingObject()
{
	if ( mpNode->GetParent() &&
		 mpNode->GetParent()->IsKindOf( CLASSINFO(WidgetNode) ) )
	{
		if ( !mpNode->GetParent()->IsKindOf( CLASSINFO(SizerNodeBase) ) )

			return &mpNode->mpView->GetGlyphForNode( *(WidgetNode*)mpNode->GetParent() );
		else
			return NULL;
	}
	else
		return NULL;
}

void WidgetGlyph::InvalidateLook()
{
	mpLookInfo = NULL;

	if ( mpBmp )
	{
		delete mpBmp;
		mpBmp = NULL; // invalidate current look-image
	}
}

/*** protected methods ***/

void WidgetGlyph::RegenerateLookBitmap()
{
	if ( !mpLookInfo ) 

		mpLookInfo = mpNode->mpView->GetWidgetDoc().FindLookForNode( *mpNode );

	if ( !mpLookInfo ) return;

	if ( mpBmp ) delete mpBmp;

	wxSize sz( mPixBounds.width, mPixBounds.height );

	mpBmp = new wxBitmap();

	WidgetEqSolver& solver = mpNode->mpView->GetWidgetDoc().GetEqSolver();

	solver.Init( mpNode->GetAttrGroupInfo( WidgetNode::WIDGET_PROPERTIES_GROUP ),
				 mpNode->GetAttrGroupData( WidgetNode::WIDGET_PROPERTIES_GROUP ),
				 mpNode->mpView->GetWidgetDoc().GetUIType() );

	bool ok = mpNode->mpView->GetWidgetDoc().GetRenderer().
		RenderItemLook( *mpLookInfo, sz, *mpBmp, GetCanvas(), &solver );
}

void WidgetGlyph::GetLimitingRectInUnits( XRect& r )
{
	mpNode->mpView->PixelRectToUnitRect( mPixBounds, r );

	if ( mpLookInfo )
	{
		r.x += mpLookInfo->mEdges[LEFT_LOOK_EDGE].mInset;
		r.y += mpLookInfo->mEdges[TOP_LOOK_EDGE].mInset;
		r.width -=  mpLookInfo->mEdges[LEFT_LOOK_EDGE].mInset +
				    mpLookInfo->mEdges[RIGHT_LOOK_EDGE].mInset;
		r.height -= mpLookInfo->mEdges[TOP_LOOK_EDGE].mInset +
				    mpLookInfo->mEdges[BOTTOM_LOOK_EDGE].mInset;
	}
}

/***** Implementation for class MyXCommandMgr *****/

MyXCommandMgr::MyXCommandMgr( wxDMCommandProcessor* pDMCmdProcessor, 
							  XCollection* pScene,
							  WidgetView* pView )

	: XCommandManager( pScene ),
	  mpMgr( pDMCmdProcessor ),
	  mpView( pView )
{}

void MyXCommandMgr::NotifyWillChange( XObject* pObj )
{
	mpMgr->NotifyWillChange( &((WidgetGlyph*)pObj)->GetNode() );
}

void MyXCommandMgr::NotifyWillRemove( XObject* pObj )
{
	mpMgr->NotifyAdded( &((WidgetGlyph*)pObj)->GetNode() );
}

void MyXCommandMgr::NotifyAdded( XObject* pObj )
{
	mpMgr->NotifyAdded( &((WidgetGlyph*)pObj)->GetNode() );
}

void MyXCommandMgr::StartBatch()
{
	mpMgr->StartBatch();
}

void MyXCommandMgr::FinishBatch()
{
	mpMgr->FinishBatch();
}

bool MyXCommandMgr::CanUndo()
{
	return mpMgr->CanUndo();
}

bool MyXCommandMgr::CanRedo()
{
	return mpMgr->CanRedo();
}

bool MyXCommandMgr::CanPaste()
{
	return mpMgr->CanPaste();
}

bool MyXCommandMgr::CanDelete()
{
	return mpMgr->CanDelete();
}

void MyXCommandMgr::Undo()
{
	mpMgr->Undo();
}

void MyXCommandMgr::Redo()
{
	mpMgr->Redo();
}

void MyXCommandMgr::Delete()
{
	mpMgr->Delete();
}

void MyXCommandMgr::Copy( XCollection* pScene )
{
	mpMgr->Copy();
}

void MyXCommandMgr::Paste( XCollection* pScene )
{
	mpMgr->Paste();
}

void MyXCommandMgr::SetCommandProcessor( wxDMCommandProcessor* pProc )
{
	mpMgr = pProc;
}

/***** Implementation for class MyXCollection *****/

MyXCollection::MyXCollection( WidgetView* pView )

	: mpView( pView ),
	  mReqsLocked( FALSE )
{}

void MyXCollection::NotifyWillChange( XObject* pObj )
{
	XCollection::NotifyWillChange( pObj );
}

void MyXCollection::NotifyWillRemove( XObject* pObj )
{
	XCollection::NotifyWillRemove( pObj );
}

void MyXCollection::NotifyAdded( XObject* pObj )
{
	XCollection::NotifyAdded( pObj );
}

void MyXCollection::SetFocus( XObject* pFocus, bool refreshNow )
{
	if ( mReqsLocked ) return;

	mReqsLocked = TRUE;

	XCollection::SetFocus( pFocus, refreshNow );

	if ( pFocus )

		((WidgetGlyph*)pFocus)->GetNode().SetFocused( TRUE );

	mReqsLocked = FALSE;
}

void MyXCollection::SetSelection( XObjListT& newSelection, bool refreshNow )
{
	if ( mReqsLocked ) return;

	mReqsLocked = TRUE;

		XCollection::SetSelection( newSelection, refreshNow );

		DoSetSelection( newSelection );

	mReqsLocked = FALSE;
}

void MyXCollection::ResetSelection()
{
	if ( mReqsLocked ) return;

	mReqsLocked = TRUE;

		XCollection::ResetSelection();

		XObjListT emptyLst;
		DoSetSelection( emptyLst );

	mReqsLocked = FALSE;
}

void MyXCollection::SelectAll()
{
	if ( mReqsLocked ) return;

	mReqsLocked = TRUE;

		XCollection::SelectAll();

		DoSetSelection( GetSelection() );

	mReqsLocked = FALSE;
}

void MyXCollection::DeleteSelection()
{
	XCollection::DeleteSelection();
}

void MyXCollection::BringSelectionToFront()
{
}

void MyXCollection::SendSelectionBack()
{
}

/*** protected methods ***/

void MyXCollection::DoSetSelection( XObjListT& lst )
{
	DMNodeListT nodeLst;

	for( size_t i = 0; i != lst.Count(); ++i )
	{
		WidgetGlyph* pGlyph = (WidgetGlyph*)lst[i];

		nodeLst.Add( &pGlyph->GetNode() );
	}

	mpView->GetRootNode().SetSelection( nodeLst );
}

/***** Implementation for class wxMyDMPasteHandler *****/

void wxMyDMPasteHandler::HandleNodePasting( wxDMNode* pNode, wxDMNode* pToNode, int depth )
{
	WidgetNode* pWNode = (WidgetNode*)pNode;
	WidgetNode* pParent = (WidgetNode*)pToNode;

	int ofs = 0;

	if ( depth == 0 ) 
		
		ofs += 20;

	pWNode->mBounds.x += + ofs;
	pWNode->mBounds.y += + ofs;
}

/***** Implementation for class WidgetInsetionEditor *****/

WidgetInsetionEditor::WidgetInsetionEditor( WidgetView* pView, int widgetCode )

	: mWigetCode( widgetCode ),
	  mpView( pView )
{
	mpPrevEditor = pView->GetScene().UnlinkEditor();

	pView->GetScene().SetEditor( this );

	wxCursor curs( wxCURSOR_HAND );
	pView->GetViewFrame()->SetCursor( curs );
}

WidgetInsetionEditor::~WidgetInsetionEditor()
{
}

void WidgetView::FindInsertionTarget( const XPoint& pos, 
									  WidgetNode*& pTarget, int& orderNo,
									  WidgetNode* pNodeToExclude )
{
	XObjListT lst = mpScene->GetContents();

	for( int i = lst.Count() - 1; i >= 0; --i )
	{
		XObject* pObj = lst[i];

		if ( pObj->GetBounds().Contains( pos ) && 
			 pObj->IsKindOf( CLASSINFO( WidgetGlyph ) ) )
		{
			WidgetNode& node = ((WidgetGlyph*)pObj)->GetNode();

			if ( &node == pNodeToExclude ) continue;

			if ( node.mpTempl->mCanHaveChildren == TRUE )
			{
				pTarget = &node;
				orderNo = -1;

				return;
			}
			else
			if ( node.GetParent() && node.GetParent()->IsKindOf( CLASSINFO( SizerNodeBase ) ) )
			{
				// otherwise we hit non-container node, find out
				// at which side of it ("left" or "right") we should 
				// insert the dragged item

				XRect bounds = pObj->GetBounds();

				orderNo = node.GetOrderNo();

				if ( pos.x >= bounds.x + bounds.width  / 2 &&
					 pos.x >= bounds.y + bounds.height / 2 )

					 orderNo++; // insert after (on the "right");

				pTarget = (WidgetNode*)node.GetParent();

				return;
			}
			// otherwise it was the node which cannot contain
			// children as well as is not surrounded by sizer-parent
		}
	}

	pTarget = NULL; // insertion target not found
}

bool WidgetInsetionEditor::OnMouseDown( const XPoint& pos )
{
	DettachFromScene();

	WidgetNode* pTarget = NULL; int zOrder = -1;

	mpView->FindInsertionTarget( pos, pTarget, zOrder, NULL );

	if ( pTarget )
	{
		XRect parentBounds = mpView->GetGlyphForNode( *pTarget ).GetBounds();

		WidgetNode* pInserted = mpView->GetWidgetDoc().
			InsertWidgetByCode( mWigetCode, pTarget, XPoint( pos.x - parentBounds.x, 
														     pos.y - parentBounds.y ),
							    zOrder );

		if ( pInserted )
		
			mpView->GetScene().SelectObject( mpView->GetGlyphForNode( *pInserted ), FALSE );

		mpView->GetScene().RefreshNow();
	}

	delete this; // destroy ourselves!

	return TRUE;
}

bool WidgetInsetionEditor::OnRightClick( const XPoint& pos )
{
	CancelCreation();

	return TRUE;
}

bool WidgetInsetionEditor::OnMouseMove( const XPoint& pos )
{
	return TRUE; // "eat" movement events
}

void WidgetInsetionEditor::CancelCreation()
{
	DettachFromScene();

	delete this; // destroy ourselves!
}

void WidgetInsetionEditor::DettachFromScene()
{
	XCollection* pScene = (XCollection*)&GetParent();

	pScene->UnlinkEditor();

	pScene->SetEditor( mpPrevEditor );

	wxCursor curs( wxCURSOR_ARROW  );
	mpView->GetViewFrame()->SetCursor( curs );
}

/***** Implementation for class GridSettingsDialog *****/

BEGIN_EVENT_TABLE( GridSettingsDialog, wxDialog )

	EVT_BUTTON( wxID_OK, GridSettingsDialog::OnOk )
	EVT_LISTBOX_DCLICK( ID_GRIT_TYPES, GridSettingsDialog::OnDblClick )

END_EVENT_TABLE()

GridSettingsDialog::GridSettingsDialog( wxWindow* pParent, 
										int gridType,
										const wxSize& gridSpacing,
										const wxSize& snapSpacing )

	: wxDialog( pParent,-1, "Grid Settings", wxDefaultPosition, wxSize( 200, 250 ),
				wxSYSTEM_MENU | wxCAPTION | 
				wxTAB_TRAVERSAL | wxTHICK_FRAME | wxRESIZE_BORDER ),
	  mGridType( gridType ),
	  mGridSpacing( gridSpacing ),
	  mSnapSpacing( snapSpacing )
{
	// create contols first
	wxString types[] = { "None", "Lines", "Dots", "Dots-3D" };
	wxString spacings[] = { "1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20" };

	mpTypes = new wxListBox( this, ID_GRIT_TYPES, wxDefaultPosition, wxDefaultSize, 4, types );
	mpGridSpacing = new wxChoice( this, -1, wxDefaultPosition, wxDefaultSize, 20, spacings );
	mpSnapSpacing = new wxChoice( this, -1, wxDefaultPosition, wxDefaultSize, 20, spacings );

	mpTypes->SetSelection( mGridType );

	if ( mGridSpacing.x >= 1 && mGridSpacing.x <= 20 )

		mpGridSpacing->SetSelection( mGridSpacing.x - 1 );
	else
		mpGridSpacing->SetSelection( 0 );

	if ( mSnapSpacing.x >= 1 && mSnapSpacing.x <= 20 )

		mpSnapSpacing->SetSelection( mSnapSpacing.x - 1 );
	else
		mpSnapSpacing->SetSelection( 0 );

	Container& cont = *new Container( this );
	cont.setInsets( Insets( 5,4,5,4 ) );
	cont.setLayout( new GridBagLayout() );

	Container* btnPanel = new Container();
	btnPanel->setInsets( Insets( 15,5,5,5 ) );
	btnPanel->setLayout( new FlowLayout() );

	wxButton* okBtn = new wxButton( this, wxID_OK, "&Ok", wxDefaultPosition );

	btnPanel->add( okBtn );
	btnPanel->add( new wxButton( this, wxID_CANCEL, "&Cancel", wxDefaultPosition ) );

	GridBagConstraints c;
	c.weightx = 1.0; c.weighty = 1.0;
	c.anchor = GridBagConstraints::WEST;
	c.insets.top = 5;
	c.insets.left = 5;

	c.weightx = 0.0; c.weighty = 1.0;
	cont.add( new wxStaticText( this,-1, "Grid type:" ), &c );
	c.fill = GridBagConstraints::BOTH;
	c.weightx = 1.0; c.gridwidth = GridBagConstraints::REMAINDER;
	cont.add( mpTypes, &c );

	c.weightx = 0.0; c.weighty = 0.0; c.gridwidth = 1;
	cont.add( new wxStaticText( this,-1, "Grid Spacing:" ), &c );
	c.weightx = 1.0; c.gridwidth = GridBagConstraints::REMAINDER;
	cont.add( mpGridSpacing, &c );

	c.weightx = 0.0; c.gridwidth = 1;
	cont.add( new wxStaticText( this,-1, "Snap Spacing:" ), &c );
	c.weightx = 1.0; c.gridwidth = GridBagConstraints::REMAINDER;
	cont.add( mpSnapSpacing, &c );

	c.weightx = 0.0;
	cont.add( btnPanel, &c );

	okBtn->SetDefault();
	mpTypes->SetFocus();
	cont.doLayout();

	Center( wxBOTH );
}
int GridSettingsDialog::GetGridType()
{
	return mGridType;
}

wxSize& GridSettingsDialog::GetGridSpacinig()
{
	return mGridSpacing;
}

wxSize& GridSettingsDialog::GetSnapSpacing()
{
	return mSnapSpacing;
}

void GridSettingsDialog::OnOk( wxCommandEvent& event )
{
	mGridSpacing.x = StrToInt( mpGridSpacing->GetStringSelection() );
	mGridSpacing.y = mGridSpacing.x;

	mSnapSpacing.x = StrToInt( mpSnapSpacing->GetStringSelection() );
	mSnapSpacing.y = mSnapSpacing.x;

	mGridType = mpTypes->GetSelection();

	EndModal( wxID_OK );
}

void GridSettingsDialog::OnDblClick( wxCommandEvent& event )
{
	OnOk( event );
}

/***** Implementation for class SizerNodeBase *****/

IMPLEMENT_CLASS( SizerNodeBase, WidgetNode )

void SizerNodeBase::GetLayoutProps( WidgetNode& node, 
								    int& option, int& flag, int& border )
{
	option = 0; flag = 0; 
	
	border = StrToInt( node.GetNodeExtraAttrValue( "border" ) );

	option = node.GetNodeExtraAttrValue( "isStrechable" ) == "true" ? 1 : 0;

	if ( node.GetNodeExtraAttrValue( "isExpandable" ) == "true" )

		flag |= wxGROW;

	if ( node.GetNodeExtraAttrValue( "hasFullBorder" ) == "true" )

		flag |= wxALL;

	if ( node.GetNodeExtraAttrValue( "hasLeftBorder" ) == "true" )

		flag |= wxLEFT;

	if ( node.GetNodeExtraAttrValue( "hasRightBorder" ) == "true" )

		flag |= wxRIGHT;

	if ( node.GetNodeExtraAttrValue( "hasTopBorder" ) == "true" )

		flag |= wxTOP;

	if ( node.GetNodeExtraAttrValue( "hasBottomBorder" ) == "true" )

		flag |= wxBOTTOM;

	wxString align = node.GetNodeExtraAttrValue( "align" );

	if ( align == "wxALIGN_CENTER" )

		flag |= wxALIGN_CENTER;
	else
	if ( align == "wxALIGN_LEFT" )

		flag |= wxALIGN_LEFT;
	else
	if ( align == "wxALIGN_RIGHT" )

		flag |= wxALIGN_RIGHT;
	else
	if ( align == "wxALIGN_BOTTOM" )

		flag |= wxALIGN_BOTTOM;
	else
	if ( align == "wxALIGN_TOP" )

		flag |= wxALIGN_TOP;
}

